@@ -3,8 +3,6 @@ package agent
 import (
 	"fmt"
 	"io"
-	"log"
-	"os"
 	"testing"
 	"time"
 
@@ -15,8 +13,9 @@ import (
 	"github.com/hashicorp/consul/agent/local"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/lib"
-	"github.com/hashicorp/consul/logger"
+	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/types"
+	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/serf/serf"
 
 	"github.com/stretchr/testify/require"
@@ -39,15 +38,12 @@ type TestACLAgent struct {
 	// to os.Stderr.
 	LogOutput io.Writer
 
-	// LogWriter is used for streaming logs.
-	LogWriter *logger.LogWriter
-
 	// DataDir is the data directory which is used when Config.DataDir
 	// is not set. It is created automatically and removed when
 	// Shutdown() is called.
 	DataDir string
 
-	resolveTokenFn func(string) (acl.Authorizer, error)
+	resolveTokenFn func(string) (structs.ACLIdentity, acl.Authorizer, error)
 
 	*Agent
 }
@@ -55,28 +51,30 @@ type TestACLAgent struct {
 // NewTestACLAGent does just enough so that all the code within agent/acl.go can work
 // Basically it needs a local state for some of the vet* functions, a logger and a delegate.
 // The key is that we are the delegate so we can control the ResolveToken responses
-func NewTestACLAgent(name string, hcl string, resolveFn func(string) (acl.Authorizer, error)) *TestACLAgent {
+func NewTestACLAgent(t *testing.T, name string, hcl string, resolveFn func(string) (structs.ACLIdentity, acl.Authorizer, error)) *TestACLAgent {
 	a := &TestACLAgent{Name: name, HCL: hcl, resolveTokenFn: resolveFn}
 	hclDataDir := `data_dir = "acl-agent"`
 
-	a.Config = TestConfig(
+	logOutput := testutil.TestWriter(t)
+	logger := hclog.NewInterceptLogger(&hclog.LoggerOptions{
+		Name:   a.Name,
+		Level:  hclog.Debug,
+		Output: logOutput,
+	})
+
+	a.Config = TestConfig(logger,
 		config.Source{Name: a.Name, Format: "hcl", Data: a.HCL},
 		config.Source{Name: a.Name + ".data_dir", Format: "hcl", Data: hclDataDir},
 	)
 
-	agent, err := New(a.Config, nil)
+	agent, err := New(a.Config, logger)
 	if err != nil {
 		panic(fmt.Sprintf("Error creating agent: %v", err))
 	}
 	a.Agent = agent
 
-	logOutput := a.LogOutput
-	if logOutput == nil {
-		logOutput = os.Stderr
-	}
 	agent.LogOutput = logOutput
-	agent.LogWriter = a.LogWriter
-	agent.logger = log.New(logOutput, a.Name+" - ", log.LstdFlags|log.Lmicroseconds)
+	agent.logger = logger
 	agent.MemSink = metrics.NewInmemSink(1*time.Second, time.Minute)
 
 	a.Agent.delegate = a
@@ -99,9 +97,51 @@ func (a *TestACLAgent) ResolveToken(secretID string) (acl.Authorizer, error) {
 		panic("This agent is useless without providing a token resolution function")
 	}
 
+	_, authz, err := a.resolveTokenFn(secretID)
+	return authz, err
+}
+
+func (a *TestACLAgent) ResolveTokenToIdentityAndAuthorizer(secretID string) (structs.ACLIdentity, acl.Authorizer, error) {
+	if a.resolveTokenFn == nil {
+		panic("This agent is useless without providing a token resolution function")
+	}
+
 	return a.resolveTokenFn(secretID)
 }
 
+func (a *TestACLAgent) ResolveTokenAndDefaultMeta(secretID string, entMeta *structs.EnterpriseMeta, authzContext *acl.AuthorizerContext) (acl.Authorizer, error) {
+	identity, authz, err := a.ResolveTokenToIdentityAndAuthorizer(secretID)
+	if err != nil {
+		return nil, err
+	}
+
+	// Default the EnterpriseMeta based on the Tokens meta or actual defaults
+	// in the case of unknown identity
+	if identity != nil {
+		entMeta.Merge(identity.EnterpriseMetadata())
+	} else {
+		entMeta.Merge(structs.DefaultEnterpriseMeta())
+	}
+
+	// Use the meta to fill in the ACL authorization context
+	entMeta.FillAuthzContext(authzContext)
+
+	return authz, err
+}
+
+func (a *TestACLAgent) ResolveIdentityFromToken(secretID string) (bool, structs.ACLIdentity, error) {
+	if a.resolveTokenFn == nil {
+		panic("This agent is useless without providing a token resolution function")
+	}
+
+	identity, _, err := a.resolveTokenFn(secretID)
+	if err != nil {
+		return true, nil, err
+	}
+
+	return true, identity, nil
+}
+
 // All of these are stubs to satisfy the interface
 func (a *TestACLAgent) Encrypted() bool {
 	return false
@@ -127,7 +167,7 @@ func (a *TestACLAgent) LocalMember() serf.Member {
 func (a *TestACLAgent) JoinLAN(addrs []string) (n int, err error) {
 	return 0, fmt.Errorf("Unimplemented")
 }
-func (a *TestACLAgent) RemoveFailedNode(node string) error {
+func (a *TestACLAgent) RemoveFailedNode(node string, prune bool) error {
 	return fmt.Errorf("Unimplemented")
 }
 
@@ -151,12 +191,12 @@ func TestACL_Version8(t *testing.T) {
 	t.Parallel()
 
 	t.Run("version 8 disabled", func(t *testing.T) {
-		resolveFn := func(string) (acl.Authorizer, error) {
+		resolveFn := func(string) (structs.ACLIdentity, acl.Authorizer, error) {
 			require.Fail(t, "should not have called delegate.ResolveToken")
-			return nil, fmt.Errorf("should not have called delegate.ResolveToken")
+			return nil, nil, fmt.Errorf("should not have called delegate.ResolveToken")
 		}
 
-		a := NewTestACLAgent(t.Name(), TestACLConfig()+`
+		a := NewTestACLAgent(t, t.Name(), TestACLConfig()+`
  		acl_enforce_version_8 = false
  	`, resolveFn)
 
@@ -167,11 +207,11 @@ func TestACL_Version8(t *testing.T) {
 
 	t.Run("version 8 enabled", func(t *testing.T) {
 		called := false
-		resolveFn := func(string) (acl.Authorizer, error) {
+		resolveFn := func(string) (structs.ACLIdentity, acl.Authorizer, error) {
 			called = true
-			return nil, acl.ErrNotFound
+			return nil, nil, acl.ErrNotFound
 		}
-		a := NewTestACLAgent(t.Name(), TestACLConfig()+`
+		a := NewTestACLAgent(t, t.Name(), TestACLConfig()+`
  		acl_enforce_version_8 = true
  	`, resolveFn)
 
@@ -184,32 +224,32 @@ func TestACL_Version8(t *testing.T) {
 func TestACL_AgentMasterToken(t *testing.T) {
 	t.Parallel()
 
-	resolveFn := func(string) (acl.Authorizer, error) {
+	resolveFn := func(string) (structs.ACLIdentity, acl.Authorizer, error) {
 		require.Fail(t, "should not have called delegate.ResolveToken")
-		return nil, fmt.Errorf("should not have called delegate.ResolveToken")
+		return nil, nil, fmt.Errorf("should not have called delegate.ResolveToken")
 	}
 
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), resolveFn)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), resolveFn)
 	a.loadTokens(a.config)
 	authz, err := a.resolveToken("towel")
 	require.NotNil(t, authz)
 	require.Nil(t, err)
 
-	require.True(t, authz.AgentRead(a.config.NodeName))
-	require.True(t, authz.AgentWrite(a.config.NodeName))
-	require.True(t, authz.NodeRead("foobarbaz"))
-	require.False(t, authz.NodeWrite("foobarbaz", nil))
+	require.Equal(t, acl.Allow, authz.AgentRead(a.config.NodeName, nil))
+	require.Equal(t, acl.Allow, authz.AgentWrite(a.config.NodeName, nil))
+	require.Equal(t, acl.Allow, authz.NodeRead("foobarbaz", nil))
+	require.Equal(t, acl.Deny, authz.NodeWrite("foobarbaz", nil))
 }
 
 func TestACL_RootAuthorizersDenied(t *testing.T) {
 	t.Parallel()
 
-	resolveFn := func(string) (acl.Authorizer, error) {
+	resolveFn := func(string) (structs.ACLIdentity, acl.Authorizer, error) {
 		require.Fail(t, "should not have called delegate.ResolveToken")
-		return nil, fmt.Errorf("should not have called delegate.ResolveToken")
+		return nil, nil, fmt.Errorf("should not have called delegate.ResolveToken")
 	}
 
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), resolveFn)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), resolveFn)
 	authz, err := a.resolveToken("deny")
 	require.Nil(t, authz)
 	require.Error(t, err)
@@ -224,63 +264,91 @@ func TestACL_RootAuthorizersDenied(t *testing.T) {
 	require.True(t, acl.IsErrRootDenied(err))
 }
 
-func authzFromPolicy(policy *acl.Policy) (acl.Authorizer, error) {
-	return acl.NewPolicyAuthorizer(acl.DenyAll(), []*acl.Policy{policy}, nil)
+func authzFromPolicy(policy *acl.Policy, cfg *acl.Config) (acl.Authorizer, error) {
+	return acl.NewPolicyAuthorizerWithDefaults(acl.DenyAll(), []*acl.Policy{policy}, cfg)
 }
 
-// catalogPolicy supplies some standard policies to help with testing the
-// catalog-related vet and filter functions.
-func catalogPolicy(token string) (acl.Authorizer, error) {
-	switch token {
+type testToken struct {
+	token structs.ACLToken
+	// yes the rules can exist on the token itself but that is legacy behavior
+	// that I would prefer these tests not rely on
+	rules string
+}
 
-	case "node-ro":
-		return authzFromPolicy(&acl.Policy{
-			NodePrefixes: []*acl.NodePolicy{
-				&acl.NodePolicy{Name: "Node", Policy: "read"},
+var (
+	nodeROSecret    = "7e80d017-bccc-492f-8dec-65f03aeaebf3"
+	nodeRWSecret    = "e3586ee5-02a2-4bf4-9ec3-9c4be7606e8c"
+	serviceROSecret = "3d2c8552-df3b-4da7-9890-36885cbf56ac"
+	serviceRWSecret = "4a1017a2-f788-4be3-93f2-90566f1340bb"
+	otherRWSecret   = "a38e8016-91b6-4876-b3e7-a307abbb2002"
+
+	testTokens = map[string]testToken{
+		nodeROSecret: testToken{
+			token: structs.ACLToken{
+				AccessorID: "9df2d1a4-2d07-414e-8ead-6053f56ed2eb",
+				SecretID:   nodeROSecret,
 			},
-		})
-	case "node-rw":
-		return authzFromPolicy(&acl.Policy{
-			NodePrefixes: []*acl.NodePolicy{
-				&acl.NodePolicy{Name: "Node", Policy: "write"},
+			rules: `node_prefix "Node" { policy = "read" }`,
+		},
+		nodeRWSecret: testToken{
+			token: structs.ACLToken{
+				AccessorID: "efb6b7d5-d343-47c1-b4cb-aa6b94d2f490",
+				SecretID:   nodeROSecret,
 			},
-		})
-	case "service-ro":
-		return authzFromPolicy(&acl.Policy{
-			ServicePrefixes: []*acl.ServicePolicy{
-				&acl.ServicePolicy{Name: "service", Policy: "read"},
+			rules: `node_prefix "Node" { policy = "write" }`,
+		},
+		serviceROSecret: testToken{
+			token: structs.ACLToken{
+				AccessorID: "0da53edb-36e5-4603-9c31-79965bad45f5",
+				SecretID:   serviceROSecret,
 			},
-		})
-	case "service-rw":
-		return authzFromPolicy(&acl.Policy{
-			ServicePrefixes: []*acl.ServicePolicy{
-				&acl.ServicePolicy{Name: "service", Policy: "write"},
+			rules: `service_prefix "service" { policy = "read" }`,
+		},
+		serviceRWSecret: testToken{
+			token: structs.ACLToken{
+				AccessorID: "52504258-137a-41e6-9326-01f40e80872e",
+				SecretID:   serviceRWSecret,
 			},
-		})
-	case "other-rw":
-		return authzFromPolicy(&acl.Policy{
-			ServicePrefixes: []*acl.ServicePolicy{
-				&acl.ServicePolicy{Name: "other", Policy: "write"},
+			rules: `service_prefix "service" { policy = "write" }`,
+		},
+		otherRWSecret: testToken{
+			token: structs.ACLToken{
+				AccessorID: "5e032c5b-c39e-4552-b5ad-8a9365b099c4",
+				SecretID:   otherRWSecret,
 			},
-		})
-	default:
-		return nil, fmt.Errorf("unknown token %q", token)
+			rules: `service_prefix "other" { policy = "write" }`,
+		},
 	}
+)
+
+func catalogPolicy(token string) (structs.ACLIdentity, acl.Authorizer, error) {
+	tok, ok := testTokens[token]
+	if !ok {
+		return nil, nil, acl.ErrNotFound
+	}
+
+	policy, err := acl.NewPolicyFromSource("", 0, tok.rules, acl.SyntaxCurrent, nil, nil)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	authz, err := authzFromPolicy(policy, nil)
+	return &tok.token, authz, err
 }
 
 func TestACL_vetServiceRegister(t *testing.T) {
 	t.Parallel()
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), catalogPolicy)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), catalogPolicy)
 
 	// Register a new service, with permission.
-	err := a.vetServiceRegister("service-rw", &structs.NodeService{
+	err := a.vetServiceRegister(serviceRWSecret, &structs.NodeService{
 		ID:      "my-service",
 		Service: "service",
 	})
 	require.NoError(t, err)
 
 	// Register a new service without write privs.
-	err = a.vetServiceRegister("service-ro", &structs.NodeService{
+	err = a.vetServiceRegister(serviceROSecret, &structs.NodeService{
 		ID:      "my-service",
 		Service: "service",
 	})
@@ -292,7 +360,7 @@ func TestACL_vetServiceRegister(t *testing.T) {
 		ID:      "my-service",
 		Service: "other",
 	}, "")
-	err = a.vetServiceRegister("service-rw", &structs.NodeService{
+	err = a.vetServiceRegister(serviceRWSecret, &structs.NodeService{
 		ID:      "my-service",
 		Service: "service",
 	})
@@ -301,10 +369,10 @@ func TestACL_vetServiceRegister(t *testing.T) {
 
 func TestACL_vetServiceUpdate(t *testing.T) {
 	t.Parallel()
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), catalogPolicy)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), catalogPolicy)
 
 	// Update a service that doesn't exist.
-	err := a.vetServiceUpdate("service-rw", "my-service")
+	err := a.vetServiceUpdate(serviceRWSecret, structs.NewServiceID("my-service", nil))
 	require.Error(t, err)
 	require.Contains(t, err.Error(), "Unknown service")
 
@@ -313,21 +381,21 @@ func TestACL_vetServiceUpdate(t *testing.T) {
 		ID:      "my-service",
 		Service: "service",
 	}, "")
-	err = a.vetServiceUpdate("service-rw", "my-service")
+	err = a.vetServiceUpdate(serviceRWSecret, structs.NewServiceID("my-service", nil))
 	require.NoError(t, err)
 
 	// Update without write privs.
-	err = a.vetServiceUpdate("service-ro", "my-service")
+	err = a.vetServiceUpdate(serviceROSecret, structs.NewServiceID("my-service", nil))
 	require.Error(t, err)
 	require.True(t, acl.IsErrPermissionDenied(err))
 }
 
 func TestACL_vetCheckRegister(t *testing.T) {
 	t.Parallel()
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), catalogPolicy)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), catalogPolicy)
 
 	// Register a new service check with write privs.
-	err := a.vetCheckRegister("service-rw", &structs.HealthCheck{
+	err := a.vetCheckRegister(serviceRWSecret, &structs.HealthCheck{
 		CheckID:     types.CheckID("my-check"),
 		ServiceID:   "my-service",
 		ServiceName: "service",
@@ -335,7 +403,7 @@ func TestACL_vetCheckRegister(t *testing.T) {
 	require.NoError(t, err)
 
 	// Register a new service check without write privs.
-	err = a.vetCheckRegister("service-ro", &structs.HealthCheck{
+	err = a.vetCheckRegister(serviceROSecret, &structs.HealthCheck{
 		CheckID:     types.CheckID("my-check"),
 		ServiceID:   "my-service",
 		ServiceName: "service",
@@ -344,13 +412,13 @@ func TestACL_vetCheckRegister(t *testing.T) {
 	require.True(t, acl.IsErrPermissionDenied(err))
 
 	// Register a new node check with write privs.
-	err = a.vetCheckRegister("node-rw", &structs.HealthCheck{
+	err = a.vetCheckRegister(nodeRWSecret, &structs.HealthCheck{
 		CheckID: types.CheckID("my-check"),
 	})
 	require.NoError(t, err)
 
 	// Register a new node check without write privs.
-	err = a.vetCheckRegister("node-ro", &structs.HealthCheck{
+	err = a.vetCheckRegister(nodeROSecret, &structs.HealthCheck{
 		CheckID: types.CheckID("my-check"),
 	})
 	require.Error(t, err)
@@ -367,7 +435,7 @@ func TestACL_vetCheckRegister(t *testing.T) {
 		ServiceID:   "my-service",
 		ServiceName: "other",
 	}, "")
-	err = a.vetCheckRegister("service-rw", &structs.HealthCheck{
+	err = a.vetCheckRegister(serviceRWSecret, &structs.HealthCheck{
 		CheckID:     types.CheckID("my-check"),
 		ServiceID:   "my-service",
 		ServiceName: "service",
@@ -379,7 +447,7 @@ func TestACL_vetCheckRegister(t *testing.T) {
 	a.State.AddCheck(&structs.HealthCheck{
 		CheckID: types.CheckID("my-node-check"),
 	}, "")
-	err = a.vetCheckRegister("service-rw", &structs.HealthCheck{
+	err = a.vetCheckRegister(serviceRWSecret, &structs.HealthCheck{
 		CheckID:     types.CheckID("my-node-check"),
 		ServiceID:   "my-service",
 		ServiceName: "service",
@@ -390,10 +458,10 @@ func TestACL_vetCheckRegister(t *testing.T) {
 
 func TestACL_vetCheckUpdate(t *testing.T) {
 	t.Parallel()
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), catalogPolicy)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), catalogPolicy)
 
 	// Update a check that doesn't exist.
-	err := a.vetCheckUpdate("node-rw", "my-check")
+	err := a.vetCheckUpdate(nodeRWSecret, structs.NewCheckID("my-check", nil))
 	require.Error(t, err)
 	require.Contains(t, err.Error(), "Unknown check")
 
@@ -407,33 +475,33 @@ func TestACL_vetCheckUpdate(t *testing.T) {
 		ServiceID:   "my-service",
 		ServiceName: "service",
 	}, "")
-	err = a.vetCheckUpdate("service-rw", "my-service-check")
+	err = a.vetCheckUpdate(serviceRWSecret, structs.NewCheckID("my-service-check", nil))
 	require.NoError(t, err)
 
 	// Update service check without write privs.
-	err = a.vetCheckUpdate("service-ro", "my-service-check")
+	err = a.vetCheckUpdate(serviceROSecret, structs.NewCheckID("my-service-check", nil))
 	require.Error(t, err)
-	require.True(t, acl.IsErrPermissionDenied(err))
+	require.True(t, acl.IsErrPermissionDenied(err), "not permission denied: %s", err.Error())
 
 	// Update node check with write privs.
 	a.State.AddCheck(&structs.HealthCheck{
 		CheckID: types.CheckID("my-node-check"),
 	}, "")
-	err = a.vetCheckUpdate("node-rw", "my-node-check")
+	err = a.vetCheckUpdate(nodeRWSecret, structs.NewCheckID("my-node-check", nil))
 	require.NoError(t, err)
 
 	// Update without write privs.
-	err = a.vetCheckUpdate("node-ro", "my-node-check")
+	err = a.vetCheckUpdate(nodeROSecret, structs.NewCheckID("my-node-check", nil))
 	require.Error(t, err)
 	require.True(t, acl.IsErrPermissionDenied(err))
 }
 
 func TestACL_filterMembers(t *testing.T) {
 	t.Parallel()
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), catalogPolicy)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), catalogPolicy)
 
 	var members []serf.Member
-	require.NoError(t, a.filterMembers("node-ro", &members))
+	require.NoError(t, a.filterMembers(nodeROSecret, &members))
 	require.Len(t, members, 0)
 
 	members = []serf.Member{
@@ -441,7 +509,7 @@ func TestACL_filterMembers(t *testing.T) {
 		serf.Member{Name: "Nope"},
 		serf.Member{Name: "Node 2"},
 	}
-	require.NoError(t, a.filterMembers("node-ro", &members))
+	require.NoError(t, a.filterMembers(nodeROSecret, &members))
 	require.Len(t, members, 2)
 	require.Equal(t, members[0].Name, "Node 1")
 	require.Equal(t, members[1].Name, "Node 2")
@@ -449,45 +517,44 @@ func TestACL_filterMembers(t *testing.T) {
 
 func TestACL_filterServices(t *testing.T) {
 	t.Parallel()
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), catalogPolicy)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), catalogPolicy)
 
-	services := make(map[string]*structs.NodeService)
-	require.NoError(t, a.filterServices("node-ro", &services))
+	services := make(map[structs.ServiceID]*structs.NodeService)
+	require.NoError(t, a.filterServices(nodeROSecret, &services))
 
-	services["my-service"] = &structs.NodeService{ID: "my-service", Service: "service"}
-	services["my-other"] = &structs.NodeService{ID: "my-other", Service: "other"}
-	require.NoError(t, a.filterServices("service-ro", &services))
-	require.Contains(t, services, "my-service")
-	require.NotContains(t, services, "my-other")
+	services[structs.NewServiceID("my-service", nil)] = &structs.NodeService{ID: "my-service", Service: "service"}
+	services[structs.NewServiceID("my-other", nil)] = &structs.NodeService{ID: "my-other", Service: "other"}
+	require.NoError(t, a.filterServices(serviceROSecret, &services))
+	require.Contains(t, services, structs.NewServiceID("my-service", nil))
+	require.NotContains(t, services, structs.NewServiceID("my-other", nil))
 }
 
 func TestACL_filterChecks(t *testing.T) {
 	t.Parallel()
-	a := NewTestACLAgent(t.Name(), TestACLConfig(), catalogPolicy)
+	a := NewTestACLAgent(t, t.Name(), TestACLConfig(), catalogPolicy)
 
-	checks := make(map[types.CheckID]*structs.HealthCheck)
-	require.NoError(t, a.filterChecks("node-ro", &checks))
+	checks := make(map[structs.CheckID]*structs.HealthCheck)
+	require.NoError(t, a.filterChecks(nodeROSecret, &checks))
 
-	checks["my-node"] = &structs.HealthCheck{}
-	checks["my-service"] = &structs.HealthCheck{ServiceName: "service"}
-	checks["my-other"] = &structs.HealthCheck{ServiceName: "other"}
-	require.NoError(t, a.filterChecks("service-ro", &checks))
-	fmt.Printf("filtered: %#v", checks)
-	_, ok := checks["my-node"]
+	checks[structs.NewCheckID("my-node", nil)] = &structs.HealthCheck{}
+	checks[structs.NewCheckID("my-service", nil)] = &structs.HealthCheck{ServiceName: "service"}
+	checks[structs.NewCheckID("my-other", nil)] = &structs.HealthCheck{ServiceName: "other"}
+	require.NoError(t, a.filterChecks(serviceROSecret, &checks))
+	_, ok := checks[structs.NewCheckID("my-node", nil)]
 	require.False(t, ok)
-	_, ok = checks["my-service"]
+	_, ok = checks[structs.NewCheckID("my-service", nil)]
 	require.True(t, ok)
-	_, ok = checks["my-other"]
+	_, ok = checks[structs.NewCheckID("my-other", nil)]
 	require.False(t, ok)
 
-	checks["my-node"] = &structs.HealthCheck{}
-	checks["my-service"] = &structs.HealthCheck{ServiceName: "service"}
-	checks["my-other"] = &structs.HealthCheck{ServiceName: "other"}
-	require.NoError(t, a.filterChecks("node-ro", &checks))
-	_, ok = checks["my-node"]
+	checks[structs.NewCheckID("my-node", nil)] = &structs.HealthCheck{}
+	checks[structs.NewCheckID("my-service", nil)] = &structs.HealthCheck{ServiceName: "service"}
+	checks[structs.NewCheckID("my-other", nil)] = &structs.HealthCheck{ServiceName: "other"}
+	require.NoError(t, a.filterChecks(nodeROSecret, &checks))
+	_, ok = checks[structs.NewCheckID("my-node", nil)]
 	require.True(t, ok)
-	_, ok = checks["my-service"]
+	_, ok = checks[structs.NewCheckID("my-service", nil)]
 	require.False(t, ok)
-	_, ok = checks["my-other"]
+	_, ok = checks[structs.NewCheckID("my-other", nil)]
 	require.False(t, ok)
 }
