@@ -292,70 +292,74 @@ func TestCatalogNodes_Blocking(t *testing.T) {
 	t.Parallel()
 	a := NewTestAgent(t, t.Name(), "")
 	defer a.Shutdown()
-	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1", testrpc.WaitForAntiEntropySync())
 
-	// Register node
+	// Run the query
 	args := &structs.DCSpecificRequest{
 		Datacenter: "dc1",
 	}
-
 	var out structs.IndexedNodes
 	if err := a.RPC("Catalog.ListNodes", *args, &out); err != nil {
 		t.Fatalf("err: %v", err)
 	}
 
-	// t.Fatal must be called from the main go routine
-	// of the test. Because of this we cannot call
-	// t.Fatal from within the go routines and use
-	// an error channel instead.
-	errch := make(chan error, 2)
+	// Async cause a change
+	waitIndex := out.Index
+	start := time.Now()
 	go func() {
-		testrpc.WaitForTestAgent(t, a.RPC, "dc1")
-		start := time.Now()
-
-		// register a service after the blocking call
-		// in order to unblock it.
-		time.AfterFunc(100*time.Millisecond, func() {
-			args := &structs.RegisterRequest{
-				Datacenter: "dc1",
-				Node:       "foo",
-				Address:    "127.0.0.1",
-			}
-			var out struct{}
-			errch <- a.RPC("Catalog.Register", args, &out)
-		})
-
-		// now block
-		req, _ := http.NewRequest("GET", fmt.Sprintf("/v1/catalog/nodes?wait=3s&index=%d", out.Index+1), nil)
-		resp := httptest.NewRecorder()
-		obj, err := a.srv.CatalogNodes(resp, req)
-		if err != nil {
-			errch <- err
+		time.Sleep(100 * time.Millisecond)
+		args := &structs.RegisterRequest{
+			Datacenter: "dc1",
+			Node:       "foo",
+			Address:    "127.0.0.1",
 		}
-
-		// Should block for a while
-		if d := time.Since(start); d < 50*time.Millisecond {
-			errch <- fmt.Errorf("too fast: %v", d)
+		var out struct{}
+		if err := a.RPC("Catalog.Register", args, &out); err != nil {
+			t.Fatalf("err: %v", err)
 		}
-
-		if idx := getIndex(t, resp); idx <= out.Index {
-			errch <- fmt.Errorf("bad: %v", idx)
-		}
-
-		nodes := obj.(structs.Nodes)
-		if len(nodes) != 2 {
-			errch <- fmt.Errorf("bad: %v", obj)
-		}
-		errch <- nil
 	}()
 
-	// wait for both go routines to return
-	if err := <-errch; err != nil {
-		t.Fatal(err)
+	const waitDuration = 3 * time.Second
+
+	// Re-run the query, if errantly woken up with no change, resume blocking.
+	var elapsed time.Duration
+RUN_BLOCKING_QUERY:
+	req, err := http.NewRequest("GET", fmt.Sprintf("/v1/catalog/nodes?wait=%s&index=%d",
+		waitDuration.String(),
+		waitIndex), nil)
+	if err != nil {
+		t.Fatalf("err: %v", err)
 	}
-	if err := <-errch; err != nil {
-		t.Fatal(err)
+	resp := httptest.NewRecorder()
+	obj, err := a.srv.CatalogNodes(resp, req)
+	if err != nil {
+		t.Fatalf("err: %v", err)
 	}
+
+	elapsed = time.Since(start)
+
+	idx := getIndex(t, resp)
+	if idx < waitIndex {
+		t.Fatalf("bad: %v", idx)
+	} else if idx == waitIndex {
+		if elapsed > waitDuration {
+			// This should prevent the loop from running longer than the
+			// waitDuration
+			t.Fatalf("too slow: %v", elapsed)
+		}
+		goto RUN_BLOCKING_QUERY
+	}
+
+	// Should block at least 100ms before getting the changed results
+	if elapsed < 100*time.Millisecond {
+		t.Fatalf("too fast: %v", elapsed)
+	}
+
+	nodes := obj.(structs.Nodes)
+	if len(nodes) != 2 {
+		t.Fatalf("bad: %v", obj)
+	}
+
 }
 
 func TestCatalogNodes_DistanceSort(t *testing.T) {
@@ -520,6 +524,59 @@ func TestCatalogServices_NodeMetaFilter(t *testing.T) {
 	}
 }
 
+func TestCatalogRegister_checkRegistration(t *testing.T) {
+	t.Parallel()
+	a := NewTestAgent(t, t.Name(), "")
+	defer a.Shutdown()
+
+	// Register node with a service and check
+	check := structs.HealthCheck{
+		Node:      "foo",
+		CheckID:   "foo-check",
+		Name:      "foo check",
+		ServiceID: "api",
+		Definition: structs.HealthCheckDefinition{
+			TCP:      "localhost:8888",
+			Interval: 5 * time.Second,
+		},
+	}
+
+	args := &structs.RegisterRequest{
+		Datacenter: "dc1",
+		Node:       "foo",
+		Address:    "127.0.0.1",
+		Service: &structs.NodeService{
+			Service: "api",
+		},
+		Check: &check,
+	}
+
+	var out struct{}
+	if err := a.RPC("Catalog.Register", args, &out); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	retry.Run(t, func(r *retry.R) {
+		req, _ := http.NewRequest("GET", "/v1/health/checks/api", nil)
+		resp := httptest.NewRecorder()
+		obj, err := a.srv.HealthServiceChecks(resp, req)
+		if err != nil {
+			r.Fatalf("err: %v", err)
+		}
+
+		checks := obj.(structs.HealthChecks)
+		if len(checks) != 1 {
+			r.Fatalf("expected 1 check, got: %d", len(checks))
+		}
+		if checks[0].CheckID != check.CheckID {
+			r.Fatalf("expected check id %s, got %s", check.Type, checks[0].Type)
+		}
+		if checks[0].Type != "tcp" {
+			r.Fatalf("expected check type tcp, got %s", checks[0].Type)
+		}
+	})
+}
+
 func TestCatalogServiceNodes(t *testing.T) {
 	t.Parallel()
 	a := NewTestAgent(t, t.Name(), "")
@@ -769,13 +826,10 @@ func TestCatalogServiceNodes_WanTranslation(t *testing.T) {
 
 	// Wait for the WAN join.
 	addr := fmt.Sprintf("127.0.0.1:%d", a1.Config.SerfPortWAN)
-	if _, err := a2.srv.agent.JoinWAN([]string{addr}); err != nil {
-		t.Fatalf("err: %v", err)
-	}
+	_, err := a2.srv.agent.JoinWAN([]string{addr})
+	require.NoError(t, err)
 	retry.Run(t, func(r *retry.R) {
-		if got, want := len(a1.WANMembers()), 2; got < want {
-			r.Fatalf("got %d WAN members want at least %d", got, want)
-		}
+		require.Len(r, a1.WANMembers(), 2)
 	})
 
 	// Register a node with DC2.
@@ -789,51 +843,51 @@ func TestCatalogServiceNodes_WanTranslation(t *testing.T) {
 			},
 			Service: &structs.NodeService{
 				Service: "http_wan_translation_test",
+				Address: "127.0.0.1",
+				Port:    8080,
+				TaggedAddresses: map[string]structs.ServiceAddress{
+					"wan": structs.ServiceAddress{
+						Address: "1.2.3.4",
+						Port:    80,
+					},
+				},
 			},
 		}
 
 		var out struct{}
-		if err := a2.RPC("Catalog.Register", args, &out); err != nil {
-			t.Fatalf("err: %v", err)
-		}
+		require.NoError(t, a2.RPC("Catalog.Register", args, &out))
 	}
 
 	// Query for the node in DC2 from DC1.
 	req, _ := http.NewRequest("GET", "/v1/catalog/service/http_wan_translation_test?dc=dc2", nil)
 	resp1 := httptest.NewRecorder()
 	obj1, err1 := a1.srv.CatalogServiceNodes(resp1, req)
-	if err1 != nil {
-		t.Fatalf("err: %v", err1)
-	}
-	assertIndex(t, resp1)
+	require.NoError(t, err1)
+	require.NoError(t, checkIndex(resp1))
 
 	// Expect that DC1 gives us a WAN address (since the node is in DC2).
-	nodes1 := obj1.(structs.ServiceNodes)
-	if len(nodes1) != 1 {
-		t.Fatalf("bad: %v", obj1)
-	}
+	nodes1, ok := obj1.(structs.ServiceNodes)
+	require.True(t, ok, "obj1 is not a structs.ServiceNodes")
+	require.Len(t, nodes1, 1)
 	node1 := nodes1[0]
-	if node1.Address != "127.0.0.2" {
-		t.Fatalf("bad: %v", node1)
-	}
+	require.Equal(t, node1.Address, "127.0.0.2")
+	require.Equal(t, node1.ServiceAddress, "1.2.3.4")
+	require.Equal(t, node1.ServicePort, 80)
 
 	// Query DC2 from DC2.
 	resp2 := httptest.NewRecorder()
 	obj2, err2 := a2.srv.CatalogServiceNodes(resp2, req)
-	if err2 != nil {
-		t.Fatalf("err: %v", err2)
-	}
-	assertIndex(t, resp2)
+	require.NoError(t, err2)
+	require.NoError(t, checkIndex(resp2))
 
 	// Expect that DC2 gives us a local address (since the node is in DC2).
-	nodes2 := obj2.(structs.ServiceNodes)
-	if len(nodes2) != 1 {
-		t.Fatalf("bad: %v", obj2)
-	}
+	nodes2, ok := obj2.(structs.ServiceNodes)
+	require.True(t, ok, "obj2 is not a structs.ServiceNodes")
+	require.Len(t, nodes2, 1)
 	node2 := nodes2[0]
-	if node2.Address != "127.0.0.1" {
-		t.Fatalf("bad: %v", node2)
-	}
+	require.Equal(t, node2.Address, "127.0.0.1")
+	require.Equal(t, node2.ServiceAddress, "127.0.0.1")
+	require.Equal(t, node2.ServicePort, 8080)
 }
 
 func TestCatalogServiceNodes_DistanceSort(t *testing.T) {
@@ -901,27 +955,28 @@ func TestCatalogServiceNodes_DistanceSort(t *testing.T) {
 	if err := a.RPC("Coordinate.Update", &arg, &out); err != nil {
 		t.Fatalf("err: %v", err)
 	}
-	time.Sleep(300 * time.Millisecond)
 
-	// Query again and now foo should have moved to the front of the line.
-	req, _ = http.NewRequest("GET", "/v1/catalog/service/api?tag=a&near=foo", nil)
-	resp = httptest.NewRecorder()
-	obj, err = a.srv.CatalogServiceNodes(resp, req)
-	if err != nil {
-		t.Fatalf("err: %v", err)
-	}
+	// Eventually foo should move to the front of the line.
+	retry.Run(t, func(r *retry.R) {
+		req, _ = http.NewRequest("GET", "/v1/catalog/service/api?tag=a&near=foo", nil)
+		resp = httptest.NewRecorder()
+		obj, err = a.srv.CatalogServiceNodes(resp, req)
+		if err != nil {
+			r.Fatalf("err: %v", err)
+		}
 
-	assertIndex(t, resp)
-	nodes = obj.(structs.ServiceNodes)
-	if len(nodes) != 2 {
-		t.Fatalf("bad: %v", obj)
-	}
-	if nodes[0].Node != "foo" {
-		t.Fatalf("bad: %v", nodes)
-	}
-	if nodes[1].Node != "bar" {
-		t.Fatalf("bad: %v", nodes)
-	}
+		assertIndex(t, resp)
+		nodes = obj.(structs.ServiceNodes)
+		if len(nodes) != 2 {
+			r.Fatalf("bad: %v", obj)
+		}
+		if nodes[0].Node != "foo" {
+			r.Fatalf("bad: %v", nodes)
+		}
+		if nodes[1].Node != "bar" {
+			r.Fatalf("bad: %v", nodes)
+		}
+	})
 }
 
 // Test that connect proxies can be queried via /v1/catalog/service/:service
@@ -950,8 +1005,6 @@ func TestCatalogServiceNodes_ConnectProxy(t *testing.T) {
 	assert.Len(nodes, 1)
 	assert.Equal(structs.ServiceKindConnectProxy, nodes[0].ServiceKind)
 	assert.Equal(args.Service.Proxy, nodes[0].ServiceProxy)
-	// DEPRECATED (ProxyDestination) - remove this when removing ProxyDestination
-	assert.Equal(args.Service.Proxy.DestinationServiceName, nodes[0].ServiceProxyDestination)
 }
 
 // Test that the Connect-compatible endpoints can be queried for a
@@ -1065,6 +1118,56 @@ func TestCatalogNodeServices(t *testing.T) {
 	require.Equal(t, args.Service.Proxy, services.Services["web-proxy"].Proxy)
 }
 
+func TestCatalogNodeServiceList(t *testing.T) {
+	t.Parallel()
+	a := NewTestAgent(t, t.Name(), "")
+	defer a.Shutdown()
+	testrpc.WaitForTestAgent(t, a.RPC, "dc1")
+
+	// Register node with a regular service and connect proxy
+	args := &structs.RegisterRequest{
+		Datacenter: "dc1",
+		Node:       "foo",
+		Address:    "127.0.0.1",
+		Service: &structs.NodeService{
+			Service: "api",
+			Tags:    []string{"a"},
+		},
+	}
+
+	var out struct{}
+	if err := a.RPC("Catalog.Register", args, &out); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	// Register a connect proxy
+	args.Service = structs.TestNodeServiceProxy(t)
+	require.NoError(t, a.RPC("Catalog.Register", args, &out))
+
+	req, _ := http.NewRequest("GET", "/v1/catalog/node-services/foo?dc=dc1", nil)
+	resp := httptest.NewRecorder()
+	obj, err := a.srv.CatalogNodeServiceList(resp, req)
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+	assertIndex(t, resp)
+
+	services := obj.(*structs.NodeServiceList)
+	if len(services.Services) != 2 {
+		t.Fatalf("bad: %v", obj)
+	}
+
+	var proxySvc *structs.NodeService
+	for _, svc := range services.Services {
+		if svc.ID == "web-proxy" {
+			proxySvc = svc
+		}
+	}
+	require.NotNil(t, proxySvc, "Missing proxy service registration")
+	// Proxy service should have it's config intact
+	require.Equal(t, args.Service.Proxy, proxySvc.Proxy)
+}
+
 func TestCatalogNodeServices_Filter(t *testing.T) {
 	t.Parallel()
 	a := NewTestAgent(t, t.Name(), "")
@@ -1150,13 +1253,10 @@ func TestCatalogNodeServices_WanTranslation(t *testing.T) {
 
 	// Wait for the WAN join.
 	addr := fmt.Sprintf("127.0.0.1:%d", a1.Config.SerfPortWAN)
-	if _, err := a2.srv.agent.JoinWAN([]string{addr}); err != nil {
-		t.Fatalf("err: %v", err)
-	}
+	_, err := a2.srv.agent.JoinWAN([]string{addr})
+	require.NoError(t, err)
 	retry.Run(t, func(r *retry.R) {
-		if got, want := len(a1.WANMembers()), 2; got < want {
-			r.Fatalf("got %d WAN members want at least %d", got, want)
-		}
+		require.Len(r, a1.WANMembers(), 2)
 	})
 
 	// Register a node with DC2.
@@ -1170,49 +1270,53 @@ func TestCatalogNodeServices_WanTranslation(t *testing.T) {
 			},
 			Service: &structs.NodeService{
 				Service: "http_wan_translation_test",
+				Address: "127.0.0.1",
+				Port:    8080,
+				TaggedAddresses: map[string]structs.ServiceAddress{
+					"wan": structs.ServiceAddress{
+						Address: "1.2.3.4",
+						Port:    80,
+					},
+				},
 			},
 		}
 
 		var out struct{}
-		if err := a2.RPC("Catalog.Register", args, &out); err != nil {
-			t.Fatalf("err: %v", err)
-		}
+		require.NoError(t, a2.RPC("Catalog.Register", args, &out))
 	}
 
 	// Query for the node in DC2 from DC1.
 	req, _ := http.NewRequest("GET", "/v1/catalog/node/foo?dc=dc2", nil)
 	resp1 := httptest.NewRecorder()
 	obj1, err1 := a1.srv.CatalogNodeServices(resp1, req)
-	if err1 != nil {
-		t.Fatalf("err: %v", err1)
-	}
-	assertIndex(t, resp1)
+	require.NoError(t, err1)
+	require.NoError(t, checkIndex(resp1))
 
 	// Expect that DC1 gives us a WAN address (since the node is in DC2).
-	services1 := obj1.(*structs.NodeServices)
-	if len(services1.Services) != 1 {
-		t.Fatalf("bad: %v", obj1)
-	}
-	service1 := services1.Node
-	if service1.Address != "127.0.0.2" {
-		t.Fatalf("bad: %v", service1)
-	}
+	service1, ok := obj1.(*structs.NodeServices)
+	require.True(t, ok, "obj1 is not a *structs.NodeServices")
+	require.NotNil(t, service1.Node)
+	require.Equal(t, service1.Node.Address, "127.0.0.2")
+	require.Len(t, service1.Services, 1)
+	ns1, ok := service1.Services["http_wan_translation_test"]
+	require.True(t, ok, "Missing service http_wan_translation_test")
+	require.Equal(t, "1.2.3.4", ns1.Address)
+	require.Equal(t, 80, ns1.Port)
 
 	// Query DC2 from DC2.
 	resp2 := httptest.NewRecorder()
 	obj2, err2 := a2.srv.CatalogNodeServices(resp2, req)
-	if err2 != nil {
-		t.Fatalf("err: %v", err2)
-	}
-	assertIndex(t, resp2)
+	require.NoError(t, err2)
+	require.NoError(t, checkIndex(resp2))
 
 	// Expect that DC2 gives us a private address (since the node is in DC2).
-	services2 := obj2.(*structs.NodeServices)
-	if len(services2.Services) != 1 {
-		t.Fatalf("bad: %v", obj2)
-	}
-	service2 := services2.Node
-	if service2.Address != "127.0.0.1" {
-		t.Fatalf("bad: %v", service2)
-	}
+	service2 := obj2.(*structs.NodeServices)
+	require.True(t, ok, "obj2 is not a *structs.NodeServices")
+	require.NotNil(t, service2.Node)
+	require.Equal(t, service2.Node.Address, "127.0.0.1")
+	require.Len(t, service2.Services, 1)
+	ns2, ok := service2.Services["http_wan_translation_test"]
+	require.True(t, ok, "Missing service http_wan_translation_test")
+	require.Equal(t, ns2.Address, "127.0.0.1")
+	require.Equal(t, ns2.Port, 8080)
 }
