@@ -4,21 +4,23 @@ import (
 	"crypto/tls"
 	"fmt"
 	"io"
-	"io/ioutil"
-	"log"
 	"net"
 	"net/http"
 	"os"
 	osexec "os/exec"
+	"strings"
 	"sync"
 	"syscall"
 	"time"
 
+	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/sdk/testutil"
+	"github.com/hashicorp/go-hclog"
+
 	"github.com/armon/circbuf"
 	"github.com/hashicorp/consul/agent/exec"
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/lib"
-	"github.com/hashicorp/consul/types"
 	"github.com/hashicorp/go-cleanhttp"
 )
 
@@ -29,7 +31,7 @@ const (
 	MinInterval = time.Second
 
 	// DefaultBufSize is the maximum size of the captured
-	// check output by defaut. Prevents an enormous buffer
+	// check output by default. Prevents an enormous buffer
 	// from being captured
 	DefaultBufSize = 4 * 1024 // 4KB
 
@@ -49,21 +51,24 @@ type RPC interface {
 // to notify when a check has a status update. The update
 // should take care to be idempotent.
 type CheckNotifier interface {
-	UpdateCheck(checkID types.CheckID, status, output string)
+	UpdateCheck(checkID structs.CheckID, status, output string)
 }
 
 // CheckMonitor is used to periodically invoke a script to
 // determine the health of a given check. It is compatible with
 // nagios plugins and expects the output in the same format.
+// Supports failures_before_critical and success_before_passing.
 type CheckMonitor struct {
 	Notify        CheckNotifier
-	CheckID       types.CheckID
+	CheckID       structs.CheckID
+	ServiceID     structs.ServiceID
 	Script        string
 	ScriptArgs    []string
 	Interval      time.Duration
 	Timeout       time.Duration
-	Logger        *log.Logger
+	Logger        hclog.Logger
 	OutputMaxSize int
+	StatusHandler *StatusHandler
 
 	stop     bool
 	stopCh   chan struct{}
@@ -117,7 +122,10 @@ func (c *CheckMonitor) check() {
 		cmd, err = exec.Script(c.Script)
 	}
 	if err != nil {
-		c.Logger.Printf("[ERR] agent: Check %q failed to setup: %s", c.CheckID, err)
+		c.Logger.Error("Check failed to setup",
+			"check", c.CheckID.String(),
+			"error", err,
+		)
 		c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, err.Error())
 		return
 	}
@@ -134,13 +142,19 @@ func (c *CheckMonitor) check() {
 			outputStr = fmt.Sprintf("Captured %d of %d bytes\n...\n%s",
 				output.Size(), output.TotalWritten(), outputStr)
 		}
-		c.Logger.Printf("[TRACE] agent: Check %q output: %s", c.CheckID, outputStr)
+		c.Logger.Trace("Check output",
+			"check", c.CheckID.String(),
+			"output", outputStr,
+		)
 		return outputStr
 	}
 
 	// Start the check
 	if err := cmd.Start(); err != nil {
-		c.Logger.Printf("[ERR] agent: Check %q failed to invoke: %s", c.CheckID, err)
+		c.Logger.Error("Check failed to invoke",
+			"check", c.CheckID.String(),
+			"error", err,
+		)
 		c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, err.Error())
 		return
 	}
@@ -158,11 +172,17 @@ func (c *CheckMonitor) check() {
 	select {
 	case <-time.After(timeout):
 		if err := exec.KillCommandSubtree(cmd); err != nil {
-			c.Logger.Printf("[WARN] agent: Check %q failed to kill after timeout: %s", c.CheckID, err)
+			c.Logger.Warn("Check failed to kill after timeout",
+				"check", c.CheckID.String(),
+				"error", err,
+			)
 		}
 
 		msg := fmt.Sprintf("Timed out (%s) running check", timeout.String())
-		c.Logger.Printf("[WARN] agent: Check %q: %s", c.CheckID, msg)
+		c.Logger.Warn("Timed out running check",
+			"check", c.CheckID.String(),
+			"timeout", timeout.String(),
+		)
 
 		outputStr := truncateAndLogOutput()
 		if len(outputStr) > 0 {
@@ -182,8 +202,7 @@ func (c *CheckMonitor) check() {
 	// Check if the check passed
 	outputStr := truncateAndLogOutput()
 	if err == nil {
-		c.Logger.Printf("[DEBUG] agent: Check %q is passing", c.CheckID)
-		c.Notify.UpdateCheck(c.CheckID, api.HealthPassing, outputStr)
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthPassing, outputStr)
 		return
 	}
 
@@ -193,16 +212,14 @@ func (c *CheckMonitor) check() {
 		if status, ok := exitErr.Sys().(syscall.WaitStatus); ok {
 			code := status.ExitStatus()
 			if code == 1 {
-				c.Logger.Printf("[WARN] agent: Check %q is now warning", c.CheckID)
-				c.Notify.UpdateCheck(c.CheckID, api.HealthWarning, outputStr)
+				c.StatusHandler.updateCheck(c.CheckID, api.HealthWarning, outputStr)
 				return
 			}
 		}
 	}
 
 	// Set the health as critical
-	c.Logger.Printf("[WARN] agent: Check %q is now critical", c.CheckID)
-	c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, outputStr)
+	c.StatusHandler.updateCheck(c.CheckID, api.HealthCritical, outputStr)
 }
 
 // CheckTTL is used to apply a TTL to check status,
@@ -210,10 +227,11 @@ func (c *CheckMonitor) check() {
 // but upon the TTL expiring, the check status is
 // automatically set to critical.
 type CheckTTL struct {
-	Notify  CheckNotifier
-	CheckID types.CheckID
-	TTL     time.Duration
-	Logger  *log.Logger
+	Notify    CheckNotifier
+	CheckID   structs.CheckID
+	ServiceID structs.ServiceID
+	TTL       time.Duration
+	Logger    hclog.Logger
 
 	timer *time.Timer
 
@@ -256,8 +274,9 @@ func (c *CheckTTL) run() {
 	for {
 		select {
 		case <-c.timer.C:
-			c.Logger.Printf("[WARN] agent: Check %q missed TTL, is now critical",
-				c.CheckID)
+			c.Logger.Warn("Check missed TTL, is now critical",
+				"check", c.CheckID.String(),
+			)
 			c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, c.getExpiredOutput())
 
 		case <-c.stopCh:
@@ -283,7 +302,10 @@ func (c *CheckTTL) getExpiredOutput() string {
 // and to renew the TTL. If expired, TTL is restarted.
 // output is returned (might be truncated)
 func (c *CheckTTL) SetStatus(status, output string) string {
-	c.Logger.Printf("[DEBUG] agent: Check %q status is now %s", c.CheckID, status)
+	c.Logger.Debug("Check status updated",
+		"check", c.CheckID.String(),
+		"status", status,
+	)
 	total := len(output)
 	if total > c.OutputMaxSize {
 		output = fmt.Sprintf("%s ... (captured %d of %d bytes)",
@@ -305,22 +327,43 @@ func (c *CheckTTL) SetStatus(status, output string) string {
 // The check is warning if the response code is 429.
 // The check is critical if the response code is anything else
 // or if the request returns an error
+// Supports failures_before_critical and success_before_passing.
 type CheckHTTP struct {
-	Notify          CheckNotifier
-	CheckID         types.CheckID
+	CheckID         structs.CheckID
+	ServiceID       structs.ServiceID
 	HTTP            string
 	Header          map[string][]string
 	Method          string
+	Body            string
 	Interval        time.Duration
 	Timeout         time.Duration
-	Logger          *log.Logger
+	Logger          hclog.Logger
 	TLSClientConfig *tls.Config
 	OutputMaxSize   int
+	StatusHandler   *StatusHandler
 
 	httpClient *http.Client
 	stop       bool
 	stopCh     chan struct{}
 	stopLock   sync.Mutex
+
+	// Set if checks are exposed through Connect proxies
+	// If set, this is the target of check()
+	ProxyHTTP string
+}
+
+func (c *CheckHTTP) CheckType() structs.CheckType {
+	return structs.CheckType{
+		CheckID:       c.CheckID.ID,
+		HTTP:          c.HTTP,
+		Method:        c.Method,
+		Body:          c.Body,
+		Header:        c.Header,
+		Interval:      c.Interval,
+		ProxyHTTP:     c.ProxyHTTP,
+		Timeout:       c.Timeout,
+		OutputMaxSize: c.OutputMaxSize,
+	}
 }
 
 // Start is used to start an HTTP check.
@@ -343,15 +386,10 @@ func (c *CheckHTTP) Start() {
 			Timeout:   10 * time.Second,
 			Transport: trans,
 		}
-
-		// For long (>10s) interval checks the http timeout is 10s, otherwise the
-		// timeout is the interval. This means that a check *should* return
-		// before the next check begins.
-		if c.Timeout > 0 && c.Timeout < c.Interval {
+		if c.Timeout > 0 {
 			c.httpClient.Timeout = c.Timeout
-		} else if c.Interval < 10*time.Second {
-			c.httpClient.Timeout = c.Interval
 		}
+
 		if c.OutputMaxSize < 1 {
 			c.OutputMaxSize = DefaultBufSize
 		}
@@ -395,10 +433,15 @@ func (c *CheckHTTP) check() {
 		method = "GET"
 	}
 
-	req, err := http.NewRequest(method, c.HTTP, nil)
+	target := c.HTTP
+	if c.ProxyHTTP != "" {
+		target = c.ProxyHTTP
+	}
+
+	bodyReader := strings.NewReader(c.Body)
+	req, err := http.NewRequest(method, target, bodyReader)
 	if err != nil {
-		c.Logger.Printf("[WARN] agent: Check %q HTTP request failed: %s", c.CheckID, err)
-		c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, err.Error())
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthCritical, err.Error())
 		return
 	}
 
@@ -422,8 +465,7 @@ func (c *CheckHTTP) check() {
 
 	resp, err := c.httpClient.Do(req)
 	if err != nil {
-		c.Logger.Printf("[WARN] agent: Check %q HTTP request failed: %s", c.CheckID, err)
-		c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, err.Error())
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthCritical, err.Error())
 		return
 	}
 	defer resp.Body.Close()
@@ -431,28 +473,26 @@ func (c *CheckHTTP) check() {
 	// Read the response into a circular buffer to limit the size
 	output, _ := circbuf.NewBuffer(int64(c.OutputMaxSize))
 	if _, err := io.Copy(output, resp.Body); err != nil {
-		c.Logger.Printf("[WARN] agent: Check %q error while reading body: %s", c.CheckID, err)
+		c.Logger.Warn("Check error while reading body",
+			"check", c.CheckID.String(),
+			"error", err,
+		)
 	}
 
 	// Format the response body
-	result := fmt.Sprintf("HTTP %s %s: %s Output: %s", method, c.HTTP, resp.Status, output.String())
+	result := fmt.Sprintf("HTTP %s %s: %s Output: %s", method, target, resp.Status, output.String())
 
 	if resp.StatusCode >= 200 && resp.StatusCode <= 299 {
 		// PASSING (2xx)
-		c.Logger.Printf("[DEBUG] agent: Check %q is passing", c.CheckID)
-		c.Notify.UpdateCheck(c.CheckID, api.HealthPassing, result)
-
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthPassing, result)
 	} else if resp.StatusCode == 429 {
 		// WARNING
 		// 429 Too Many Requests (RFC 6585)
 		// The user has sent too many requests in a given amount of time.
-		c.Logger.Printf("[WARN] agent: Check %q is now warning", c.CheckID)
-		c.Notify.UpdateCheck(c.CheckID, api.HealthWarning, result)
-
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthWarning, result)
 	} else {
 		// CRITICAL
-		c.Logger.Printf("[WARN] agent: Check %q is now critical", c.CheckID)
-		c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, result)
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthCritical, result)
 	}
 }
 
@@ -460,13 +500,15 @@ func (c *CheckHTTP) check() {
 // determine the health of a given check.
 // The check is passing if the connection succeeds
 // The check is critical if the connection returns an error
+// Supports failures_before_critical and success_before_passing.
 type CheckTCP struct {
-	Notify   CheckNotifier
-	CheckID  types.CheckID
-	TCP      string
-	Interval time.Duration
-	Timeout  time.Duration
-	Logger   *log.Logger
+	CheckID       structs.CheckID
+	ServiceID     structs.ServiceID
+	TCP           string
+	Interval      time.Duration
+	Timeout       time.Duration
+	Logger        hclog.Logger
+	StatusHandler *StatusHandler
 
 	dialer   *net.Dialer
 	stop     bool
@@ -482,15 +524,12 @@ func (c *CheckTCP) Start() {
 
 	if c.dialer == nil {
 		// Create the socket dialer
-		c.dialer = &net.Dialer{DualStack: true}
-
-		// For long (>10s) interval checks the socket timeout is 10s, otherwise
-		// the timeout is the interval. This means that a check *should* return
-		// before the next check begins.
-		if c.Timeout > 0 && c.Timeout < c.Interval {
+		c.dialer = &net.Dialer{
+			Timeout:   10 * time.Second,
+			DualStack: true,
+		}
+		if c.Timeout > 0 {
 			c.dialer.Timeout = c.Timeout
-		} else if c.Interval < 10*time.Second {
-			c.dialer.Timeout = c.Interval
 		}
 	}
 
@@ -529,29 +568,33 @@ func (c *CheckTCP) run() {
 func (c *CheckTCP) check() {
 	conn, err := c.dialer.Dial(`tcp`, c.TCP)
 	if err != nil {
-		c.Logger.Printf("[WARN] agent: Check %q socket connection failed: %s", c.CheckID, err)
-		c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, err.Error())
+		c.Logger.Warn("Check socket connection failed",
+			"check", c.CheckID.String(),
+			"error", err,
+		)
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthCritical, err.Error())
 		return
 	}
 	conn.Close()
-	c.Logger.Printf("[DEBUG] agent: Check %q is passing", c.CheckID)
-	c.Notify.UpdateCheck(c.CheckID, api.HealthPassing, fmt.Sprintf("TCP connect %s: Success", c.TCP))
+	c.StatusHandler.updateCheck(c.CheckID, api.HealthPassing, fmt.Sprintf("TCP connect %s: Success", c.TCP))
 }
 
 // CheckDocker is used to periodically invoke a script to
 // determine the health of an application running inside a
 // Docker Container. We assume that the script is compatible
 // with nagios plugins and expects the output in the same format.
+// Supports failures_before_critical and success_before_passing.
 type CheckDocker struct {
-	Notify            CheckNotifier
-	CheckID           types.CheckID
+	CheckID           structs.CheckID
+	ServiceID         structs.ServiceID
 	Script            string
 	ScriptArgs        []string
 	DockerContainerID string
 	Shell             string
 	Interval          time.Duration
-	Logger            *log.Logger
+	Logger            hclog.Logger
 	Client            *DockerClient
+	StatusHandler     *StatusHandler
 
 	stop chan struct{}
 }
@@ -562,7 +605,7 @@ func (c *CheckDocker) Start() {
 	}
 
 	if c.Logger == nil {
-		c.Logger = log.New(ioutil.Discard, "", 0)
+		c.Logger = testutil.NewDiscardLogger()
 	}
 
 	if c.Shell == "" {
@@ -601,7 +644,10 @@ func (c *CheckDocker) check() {
 	var out string
 	status, b, err := c.doCheck()
 	if err != nil {
-		c.Logger.Printf("[DEBUG] agent: Check %q: %s", c.CheckID, err)
+		c.Logger.Debug("Check failed",
+			"check", c.CheckID.String(),
+			"error", err,
+		)
 		out = err.Error()
 	} else {
 		// out is already limited to CheckBufSize since we're getting a
@@ -611,14 +657,12 @@ func (c *CheckDocker) check() {
 		if int(b.TotalWritten()) > len(out) {
 			out = fmt.Sprintf("Captured %d of %d bytes\n...\n%s", len(out), b.TotalWritten(), out)
 		}
-		c.Logger.Printf("[TRACE] agent: Check %q output: %s", c.CheckID, out)
+		c.Logger.Trace("Check output",
+			"check", c.CheckID.String(),
+			"output", out,
+		)
 	}
-
-	if status == api.HealthCritical {
-		c.Logger.Printf("[WARN] agent: Check %q is now critical", c.CheckID)
-	}
-
-	c.Notify.UpdateCheck(c.CheckID, status, out)
+	c.StatusHandler.updateCheck(c.CheckID, status, out)
 }
 
 func (c *CheckDocker) doCheck() (string, *circbuf.Buffer, error) {
@@ -648,10 +692,16 @@ func (c *CheckDocker) doCheck() (string, *circbuf.Buffer, error) {
 	case 0:
 		return api.HealthPassing, buf, nil
 	case 1:
-		c.Logger.Printf("[DEBUG] agent: Check %q failed with exit code: %d", c.CheckID, exitCode)
+		c.Logger.Debug("Check failed",
+			"check", c.CheckID.String(),
+			"exit_code", exitCode,
+		)
 		return api.HealthWarning, buf, nil
 	default:
-		c.Logger.Printf("[DEBUG] agent: Check %q failed with exit code: %d", c.CheckID, exitCode)
+		c.Logger.Debug("Check failed",
+			"check", c.CheckID.String(),
+			"exit_code", exitCode,
+		)
 		return api.HealthCritical, buf, nil
 	}
 }
@@ -661,19 +711,35 @@ func (c *CheckDocker) doCheck() (string, *circbuf.Buffer, error) {
 // The check is passing if returned status is SERVING.
 // The check is critical if connection fails or returned status is
 // not SERVING.
+// Supports failures_before_critical and success_before_passing.
 type CheckGRPC struct {
-	Notify          CheckNotifier
-	CheckID         types.CheckID
+	CheckID         structs.CheckID
+	ServiceID       structs.ServiceID
 	GRPC            string
 	Interval        time.Duration
 	Timeout         time.Duration
 	TLSClientConfig *tls.Config
-	Logger          *log.Logger
+	Logger          hclog.Logger
+	StatusHandler   *StatusHandler
 
 	probe    *GrpcHealthProbe
 	stop     bool
 	stopCh   chan struct{}
 	stopLock sync.Mutex
+
+	// Set if checks are exposed through Connect proxies
+	// If set, this is the target of check()
+	ProxyGRPC string
+}
+
+func (c *CheckGRPC) CheckType() structs.CheckType {
+	return structs.CheckType{
+		CheckID:   c.CheckID.ID,
+		GRPC:      c.GRPC,
+		ProxyGRPC: c.ProxyGRPC,
+		Interval:  c.Interval,
+		Timeout:   c.Timeout,
+	}
 }
 
 func (c *CheckGRPC) Start() {
@@ -705,13 +771,16 @@ func (c *CheckGRPC) run() {
 }
 
 func (c *CheckGRPC) check() {
-	err := c.probe.Check()
+	target := c.GRPC
+	if c.ProxyGRPC != "" {
+		target = c.ProxyGRPC
+	}
+
+	err := c.probe.Check(target)
 	if err != nil {
-		c.Logger.Printf("[DEBUG] agent: Check %q failed: %s", c.CheckID, err.Error())
-		c.Notify.UpdateCheck(c.CheckID, api.HealthCritical, err.Error())
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthCritical, err.Error())
 	} else {
-		c.Logger.Printf("[DEBUG] agent: Check %q is passing", c.CheckID)
-		c.Notify.UpdateCheck(c.CheckID, api.HealthPassing, fmt.Sprintf("gRPC check %s: success", c.GRPC))
+		c.StatusHandler.updateCheck(c.CheckID, api.HealthPassing, fmt.Sprintf("gRPC check %s: success", target))
 	}
 }
 
@@ -723,3 +792,63 @@ func (c *CheckGRPC) Stop() {
 		close(c.stopCh)
 	}
 }
+
+// StatusHandler keep tracks of successive error/success counts and ensures
+// that status can be set to critical/passing only once the successive number of event
+// reaches the given threshold.
+type StatusHandler struct {
+	inner                  CheckNotifier
+	logger                 hclog.Logger
+	successBeforePassing   int
+	successCounter         int
+	failuresBeforeCritical int
+	failuresCounter        int
+}
+
+// NewStatusHandler set counters values to threshold in order to immediatly update status after first check.
+func NewStatusHandler(inner CheckNotifier, logger hclog.Logger, successBeforePassing, failuresBeforeCritical int) *StatusHandler {
+	return &StatusHandler{
+		logger:                 logger,
+		inner:                  inner,
+		successBeforePassing:   successBeforePassing,
+		successCounter:         successBeforePassing,
+		failuresBeforeCritical: failuresBeforeCritical,
+		failuresCounter:        failuresBeforeCritical,
+	}
+}
+
+func (s *StatusHandler) updateCheck(checkID structs.CheckID, status, output string) {
+
+	if status == api.HealthPassing || status == api.HealthWarning {
+		s.successCounter++
+		s.failuresCounter = 0
+		if s.successCounter >= s.successBeforePassing {
+			s.logger.Debug("Check status updated",
+				"check", checkID.String(),
+				"status", status,
+			)
+			s.inner.UpdateCheck(checkID, status, output)
+			return
+		}
+		s.logger.Warn("Check passed but has not reached success threshold",
+			"check", checkID.String(),
+			"status", status,
+			"success_count", s.successCounter,
+			"success_threshold", s.successBeforePassing,
+		)
+	} else {
+		s.failuresCounter++
+		s.successCounter = 0
+		if s.failuresCounter >= s.failuresBeforeCritical {
+			s.logger.Warn("Check is now critical", "check", checkID.String())
+			s.inner.UpdateCheck(checkID, status, output)
+			return
+		}
+		s.logger.Warn("Check failed but has not reached failure threshold",
+			"check", checkID.String(),
+			"status", status,
+			"failure_count", s.failuresCounter,
+			"failure_threshold", s.failuresBeforeCritical,
+		)
+	}
+}
