@@ -9,6 +9,7 @@ import (
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/hashicorp/raft"
 	"github.com/hashicorp/serf/serf"
+	"github.com/stretchr/testify/require"
 )
 
 func TestAutopilot_IdempotentShutdown(t *testing.T) {
@@ -37,7 +38,7 @@ func testCleanupDeadServer(t *testing.T, raftVersion int) {
 	conf := func(c *Config) {
 		c.Datacenter = dc
 		c.Bootstrap = false
-		c.BootstrapExpect = 3
+		c.BootstrapExpect = 5
 		c.RaftConfig.ProtocolVersion = raft.ProtocolVersion(raftVersion)
 	}
 	dir1, s1 := testServerWithConfig(t, conf)
@@ -52,43 +53,68 @@ func testCleanupDeadServer(t *testing.T, raftVersion int) {
 	defer os.RemoveAll(dir3)
 	defer s3.Shutdown()
 
-	servers := []*Server{s1, s2, s3}
-
-	// Try to join
-	joinLAN(t, s2, s1)
-	joinLAN(t, s3, s1)
-
-	for _, s := range servers {
-		testrpc.WaitForLeader(t, s.RPC, dc)
-		retry.Run(t, func(r *retry.R) { r.Check(wantPeers(s, 3)) })
-	}
-
-	// Bring up a new server
 	dir4, s4 := testServerWithConfig(t, conf)
 	defer os.RemoveAll(dir4)
 	defer s4.Shutdown()
 
-	// Kill a non-leader server
-	s3.Shutdown()
+	dir5, s5 := testServerWithConfig(t, conf)
+	defer os.RemoveAll(dir5)
+	defer s5.Shutdown()
+
+	servers := []*Server{s1, s2, s3, s4, s5}
+
+	// Try to join
+	joinLAN(t, s2, s1)
+	joinLAN(t, s3, s1)
+	joinLAN(t, s4, s1)
+	joinLAN(t, s5, s1)
+
+	for _, s := range servers {
+		testrpc.WaitForLeader(t, s.RPC, dc)
+		retry.Run(t, func(r *retry.R) { r.Check(wantPeers(s, 5)) })
+	}
+
+	require := require.New(t)
+	testrpc.WaitForLeader(t, s1.RPC, "dc1")
+	leaderIndex := -1
+	for i, s := range servers {
+		if s.IsLeader() {
+			leaderIndex = i
+			break
+		}
+	}
+	require.NotEqual(leaderIndex, -1)
+
+	// Shutdown two non-leader servers
+	killed := make(map[string]struct{})
+	for i, s := range servers {
+		if i != leaderIndex {
+			s.Shutdown()
+			killed[string(s.config.NodeID)] = struct{}{}
+		}
+		if len(killed) == 2 {
+			break
+		}
+	}
+
 	retry.Run(t, func(r *retry.R) {
 		alive := 0
-		for _, m := range s1.LANMembers() {
+		for _, m := range servers[leaderIndex].LANMembers() {
 			if m.Status == serf.StatusAlive {
 				alive++
 			}
 		}
-		if alive != 2 {
-			r.Fatal(nil)
+		if alive != 3 {
+			r.Fatalf("Expected three alive servers instead of %d", alive)
 		}
 	})
 
-	// Join the new server
-	joinLAN(t, s4, s1)
-	servers[2] = s4
-
-	// Make sure the dead server is removed and we're back to 3 total peers
+	// Make sure the dead servers are removed and we're back to 3 total peers
 	for _, s := range servers {
-		retry.Run(t, func(r *retry.R) { r.Check(wantPeers(s, 3)) })
+		_, killed := killed[string(s.config.NodeID)]
+		if !killed {
+			retry.Run(t, func(r *retry.R) { r.Check(wantPeers(s, 3)) })
+		}
 	}
 }
 
@@ -284,7 +310,10 @@ func TestAutopilot_CleanupStaleRaftServer(t *testing.T) {
 	testrpc.WaitForLeader(t, s1.RPC, "dc1")
 
 	// Add s4 to peers directly
-	s1.raft.AddVoter(raft.ServerID(s4.config.NodeID), raft.ServerAddress(joinAddrLAN(s4)), 0, 0)
+	addVoterFuture := s1.raft.AddVoter(raft.ServerID(s4.config.NodeID), raft.ServerAddress(joinAddrLAN(s4)), 0, 0)
+	if err := addVoterFuture.Error(); err != nil {
+		t.Fatal(err)
+	}
 
 	// Verify we have 4 peers
 	peers, err := s1.numPeers()
@@ -369,3 +398,103 @@ func TestAutopilot_PromoteNonVoter(t *testing.T) {
 		}
 	})
 }
+
+func TestAutopilot_MinQuorum(t *testing.T) {
+	dc := "dc1"
+	closeMap := make(map[string]chan struct{})
+	conf := func(c *Config) {
+		c.Datacenter = dc
+		c.Bootstrap = false
+		c.BootstrapExpect = 4
+		c.AutopilotConfig.MinQuorum = 3
+		c.RaftConfig.ProtocolVersion = raft.ProtocolVersion(2)
+		c.AutopilotInterval = 100 * time.Millisecond
+		//Let us know when a server is actually gone
+		ch := make(chan struct{})
+		c.NotifyShutdown = func() {
+			t.Logf("%v is shutdown", c.NodeName)
+			close(ch)
+		}
+		closeMap[c.NodeName] = ch
+	}
+	dir1, s1 := testServerWithConfig(t, conf)
+	defer os.RemoveAll(dir1)
+	defer s1.Shutdown()
+
+	dir2, s2 := testServerWithConfig(t, conf)
+	defer os.RemoveAll(dir2)
+	defer s2.Shutdown()
+
+	dir3, s3 := testServerWithConfig(t, conf)
+	defer os.RemoveAll(dir3)
+	defer s3.Shutdown()
+
+	dir4, s4 := testServerWithConfig(t, conf)
+	defer os.RemoveAll(dir4)
+	defer s4.Shutdown()
+
+	servers := map[string]*Server{s1.config.NodeName: s1,
+		s2.config.NodeName: s2,
+		s3.config.NodeName: s3,
+		s4.config.NodeName: s4}
+
+	// Try to join
+	joinLAN(t, s2, s1)
+	joinLAN(t, s3, s1)
+	joinLAN(t, s4, s1)
+
+	//Differentiate between leader and server
+	findStatus := func(leader bool) *Server {
+		for _, mem := range servers {
+			if mem.IsLeader() == leader {
+				return mem
+			}
+			if !mem.IsLeader() == !leader {
+				return mem
+			}
+		}
+
+		return nil
+	}
+	testrpc.WaitForLeader(t, s1.RPC, dc)
+
+	// Have autopilot take one into left
+	dead := findStatus(false)
+	if dead == nil {
+		t.Fatalf("no members set")
+	}
+	dead.Shutdown()
+	<-closeMap[dead.config.NodeName]
+	retry.Run(t, func(r *retry.R) {
+		leader := findStatus(true)
+		if leader == nil {
+			r.Fatalf("no members set")
+		}
+		for _, m := range leader.LANMembers() {
+			if m.Name == dead.config.NodeName && m.Status != serf.StatusLeft {
+				r.Fatalf("%v should be left, got %v", m.Name, m.Status.String())
+			}
+		}
+	})
+
+	delete(servers, dead.config.NodeName)
+	//Autopilot should not take this one into left
+	dead = findStatus(false)
+	if err := dead.Shutdown(); err != nil {
+		t.Fatalf("could not shut down %s, error %v", dead.config.NodeName, err)
+	}
+	<-closeMap[dead.config.NodeName]
+
+	retry.Run(t, func(r *retry.R) {
+		leader := findStatus(true)
+		if leader == nil {
+			r.Fatalf("no members set")
+		}
+		for _, m := range leader.LANMembers() {
+			if m.Name == dead.config.NodeName && m.Status != serf.StatusFailed {
+				r.Fatalf("%v should be failed, got %v", m.Name, m.Status.String())
+			}
+		}
+	})
+
+}
