@@ -35,7 +35,7 @@ func nodePreApply(nodeName, nodeID string) error {
 	return nil
 }
 
-func servicePreApply(service *structs.NodeService, rule acl.Authorizer) error {
+func servicePreApply(service *structs.NodeService, authz acl.Authorizer) error {
 	// Validate the service. This is in addition to the below since
 	// the above just hasn't been moved over yet. We should move it over
 	// in time.
@@ -59,20 +59,23 @@ func servicePreApply(service *structs.NodeService, rule acl.Authorizer) error {
 		return fmt.Errorf("Invalid service address")
 	}
 
+	var authzContext acl.AuthorizerContext
+	service.FillAuthzContext(&authzContext)
+
 	// Apply the ACL policy if any. The 'consul' service is excluded
 	// since it is managed automatically internally (that behavior
 	// is going away after version 0.8). We check this same policy
 	// later if version 0.8 is enabled, so we can eventually just
 	// delete this and do all the ACL checks down there.
 	if service.Service != structs.ConsulServiceName {
-		if rule != nil && !rule.ServiceWrite(service.Service, nil) {
+		if authz != nil && authz.ServiceWrite(service.Service, &authzContext) != acl.Allow {
 			return acl.ErrPermissionDenied
 		}
 	}
 
 	// Proxies must have write permission on their destination
 	if service.Kind == structs.ServiceKindConnectProxy {
-		if rule != nil && !rule.ServiceWrite(service.Proxy.DestinationServiceName, nil) {
+		if authz != nil && authz.ServiceWrite(service.Proxy.DestinationServiceName, &authzContext) != acl.Allow {
 			return acl.ErrPermissionDenied
 		}
 	}
@@ -95,7 +98,19 @@ func (c *Catalog) Register(args *structs.RegisterRequest, reply *struct{}) error
 	defer metrics.MeasureSince([]string{"catalog", "register"}, time.Now())
 
 	// Fetch the ACL token, if any.
-	rule, err := c.srv.ResolveToken(args.Token)
+	authz, err := c.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, nil)
+	if err != nil {
+		return err
+	}
+
+	if err := c.srv.validateEnterpriseRequest(args.GetEnterpriseMeta(), true); err != nil {
+		return err
+	}
+
+	// This needs to happen before the other preapply checks as it will fixup some of the
+	// internal enterprise metas on the services and checks
+	state := c.srv.fsm.State()
+	entMeta, err := state.ValidateRegisterRequest(args)
 	if err != nil {
 		return err
 	}
@@ -110,7 +125,7 @@ func (c *Catalog) Register(args *structs.RegisterRequest, reply *struct{}) error
 
 	// Handle a service registration.
 	if args.Service != nil {
-		if err := servicePreApply(args.Service, rule); err != nil {
+		if err := servicePreApply(args.Service, authz); err != nil {
 			return err
 		}
 	}
@@ -125,16 +140,23 @@ func (c *Catalog) Register(args *structs.RegisterRequest, reply *struct{}) error
 			check.Node = args.Node
 		}
 		checkPreApply(check)
+
+		// Populate check type for cases when a check is registered in the catalog directly
+		// and not via anti-entropy
+		if check.Type == "" {
+			chkType := check.CheckType()
+			check.Type = chkType.Type()
+		}
 	}
 
 	// Check the complete register request against the given ACL policy.
-	if rule != nil && c.srv.config.ACLEnforceVersion8 {
+	if authz != nil && c.srv.config.ACLEnforceVersion8 {
 		state := c.srv.fsm.State()
-		_, ns, err := state.NodeServices(nil, args.Node)
+		_, ns, err := state.NodeServices(nil, args.Node, entMeta)
 		if err != nil {
 			return fmt.Errorf("Node lookup failed: %v", err)
 		}
-		if err := vetRegisterWithACL(rule, args, ns); err != nil {
+		if err := vetRegisterWithACL(authz, args, ns); err != nil {
 			return err
 		}
 	}
@@ -162,18 +184,22 @@ func (c *Catalog) Deregister(args *structs.DeregisterRequest, reply *struct{}) e
 	}
 
 	// Fetch the ACL token, if any.
-	rule, err := c.srv.ResolveToken(args.Token)
+	authz, err := c.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, nil)
 	if err != nil {
 		return err
 	}
 
+	if err := c.srv.validateEnterpriseRequest(&args.EnterpriseMeta, true); err != nil {
+		return err
+	}
+
 	// Check the complete deregister request against the given ACL policy.
-	if rule != nil && c.srv.config.ACLEnforceVersion8 {
+	if authz != nil && c.srv.config.ACLEnforceVersion8 {
 		state := c.srv.fsm.State()
 
 		var ns *structs.NodeService
 		if args.ServiceID != "" {
-			_, ns, err = state.NodeService(args.Node, args.ServiceID)
+			_, ns, err = state.NodeService(args.Node, args.ServiceID, &args.EnterpriseMeta)
 			if err != nil {
 				return fmt.Errorf("Service lookup failed: %v", err)
 			}
@@ -181,13 +207,13 @@ func (c *Catalog) Deregister(args *structs.DeregisterRequest, reply *struct{}) e
 
 		var nc *structs.HealthCheck
 		if args.CheckID != "" {
-			_, nc, err = state.NodeCheck(args.Node, args.CheckID)
+			_, nc, err = state.NodeCheck(args.Node, args.CheckID, &args.EnterpriseMeta)
 			if err != nil {
 				return fmt.Errorf("Check lookup failed: %v", err)
 			}
 		}
 
-		if err := vetDeregisterWithACL(rule, args, ns, nc); err != nil {
+		if err := vetDeregisterWithACL(authz, args, ns, nc); err != nil {
 			return err
 		}
 
@@ -200,7 +226,7 @@ func (c *Catalog) Deregister(args *structs.DeregisterRequest, reply *struct{}) e
 }
 
 // ListDatacenters is used to query for the list of known datacenters
-func (c *Catalog) ListDatacenters(args *struct{}, reply *[]string) error {
+func (c *Catalog) ListDatacenters(args *structs.DatacentersRequest, reply *[]string) error {
 	dcs, err := c.srv.router.GetDatacentersByDistance()
 	if err != nil {
 		return err
@@ -262,6 +288,17 @@ func (c *Catalog) ListServices(args *structs.DCSpecificRequest, reply *structs.I
 		return err
 	}
 
+	(*reply).EnterpriseMeta = args.EnterpriseMeta
+
+	authz, err := c.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, nil)
+	if err != nil {
+		return err
+	}
+
+	if err := c.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
 	return c.srv.blockingQuery(
 		&args.QueryOptions,
 		&reply.QueryMeta,
@@ -270,16 +307,44 @@ func (c *Catalog) ListServices(args *structs.DCSpecificRequest, reply *structs.I
 			var services structs.Services
 			var err error
 			if len(args.NodeMetaFilters) > 0 {
-				index, services, err = state.ServicesByNodeMeta(ws, args.NodeMetaFilters)
+				index, services, err = state.ServicesByNodeMeta(ws, args.NodeMetaFilters, &args.EnterpriseMeta)
 			} else {
-				index, services, err = state.Services(ws)
+				index, services, err = state.Services(ws, &args.EnterpriseMeta)
 			}
 			if err != nil {
 				return err
 			}
 
+			reply.Index, reply.Services, reply.EnterpriseMeta = index, services, args.EnterpriseMeta
+			return c.srv.filterACLWithAuthorizer(authz, reply)
+		})
+}
+
+func (c *Catalog) ServiceList(args *structs.DCSpecificRequest, reply *structs.IndexedServiceList) error {
+	if done, err := c.srv.forward("Catalog.ServiceList", args, args, reply); done {
+		return err
+	}
+
+	authz, err := c.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, nil)
+	if err != nil {
+		return err
+	}
+
+	if err := c.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
+	return c.srv.blockingQuery(
+		&args.QueryOptions,
+		&reply.QueryMeta,
+		func(ws memdb.WatchSet, state *state.Store) error {
+			index, services, err := state.ServiceList(ws, &args.EnterpriseMeta)
+			if err != nil {
+				return err
+			}
+
 			reply.Index, reply.Services = index, services
-			return c.srv.filterACL(args.Token, reply)
+			return c.srv.filterACLWithAuthorizer(authz, reply)
 		})
 }
 
@@ -299,13 +364,13 @@ func (c *Catalog) ServiceNodes(args *structs.ServiceSpecificRequest, reply *stru
 	switch {
 	case args.Connect:
 		f = func(ws memdb.WatchSet, s *state.Store) (uint64, structs.ServiceNodes, error) {
-			return s.ConnectServiceNodes(ws, args.ServiceName)
+			return s.ConnectServiceNodes(ws, args.ServiceName, &args.EnterpriseMeta)
 		}
 
 	default:
 		f = func(ws memdb.WatchSet, s *state.Store) (uint64, structs.ServiceNodes, error) {
 			if args.ServiceAddress != "" {
-				return s.ServiceAddressNodes(ws, args.ServiceAddress)
+				return s.ServiceAddressNodes(ws, args.ServiceAddress, &args.EnterpriseMeta)
 			}
 
 			if args.TagFilter {
@@ -318,23 +383,27 @@ func (c *Catalog) ServiceNodes(args *structs.ServiceSpecificRequest, reply *stru
 					tags = []string{args.ServiceTag}
 				}
 
-				return s.ServiceTagNodes(ws, args.ServiceName, tags)
+				return s.ServiceTagNodes(ws, args.ServiceName, tags, &args.EnterpriseMeta)
 			}
 
-			return s.ServiceNodes(ws, args.ServiceName)
+			return s.ServiceNodes(ws, args.ServiceName, &args.EnterpriseMeta)
 		}
 	}
 
+	var authzContext acl.AuthorizerContext
+	authz, err := c.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, &authzContext)
+	if err != nil {
+		return err
+	}
+
+	if err := c.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
 	// If we're doing a connect query, we need read access to the service
 	// we're trying to find proxies for, so check that.
 	if args.Connect {
-		// Fetch the ACL token, if any.
-		rule, err := c.srv.ResolveToken(args.Token)
-		if err != nil {
-			return err
-		}
-
-		if rule != nil && !rule.ServiceRead(args.ServiceName) {
+		if authz != nil && authz.ServiceRead(args.ServiceName, &authzContext) != acl.Allow {
 			// Just return nil, which will return an empty response (tested)
 			return nil
 		}
@@ -434,11 +503,20 @@ func (c *Catalog) NodeServices(args *structs.NodeSpecificRequest, reply *structs
 		return err
 	}
 
+	_, err = c.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, nil)
+	if err != nil {
+		return err
+	}
+
+	if err := c.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
 	return c.srv.blockingQuery(
 		&args.QueryOptions,
 		&reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, services, err := state.NodeServices(ws, args.Node)
+			index, services, err := state.NodeServices(ws, args.Node, &args.EnterpriseMeta)
 			if err != nil {
 				return err
 			}
@@ -459,3 +537,56 @@ func (c *Catalog) NodeServices(args *structs.NodeSpecificRequest, reply *structs
 			return nil
 		})
 }
+
+func (c *Catalog) NodeServiceList(args *structs.NodeSpecificRequest, reply *structs.IndexedNodeServiceList) error {
+	if done, err := c.srv.forward("Catalog.NodeServiceList", args, args, reply); done {
+		return err
+	}
+
+	// Verify the arguments
+	if args.Node == "" {
+		return fmt.Errorf("Must provide node")
+	}
+
+	var filterType []*structs.NodeService
+	filter, err := bexpr.CreateFilter(args.Filter, nil, filterType)
+	if err != nil {
+		return err
+	}
+
+	_, err = c.srv.ResolveTokenAndDefaultMeta(args.Token, &args.EnterpriseMeta, nil)
+	if err != nil {
+		return err
+	}
+
+	if err := c.srv.validateEnterpriseRequest(&args.EnterpriseMeta, false); err != nil {
+		return err
+	}
+
+	return c.srv.blockingQuery(
+		&args.QueryOptions,
+		&reply.QueryMeta,
+		func(ws memdb.WatchSet, state *state.Store) error {
+			index, services, err := state.NodeServiceList(ws, args.Node, &args.EnterpriseMeta)
+			if err != nil {
+				return err
+			}
+
+			if err := c.srv.filterACL(args.Token, &services); err != nil {
+				return err
+			}
+
+			reply.Index = index
+			if services != nil {
+				reply.NodeServices = *services
+
+				raw, err := filter.Execute(reply.NodeServices.Services)
+				if err != nil {
+					return err
+				}
+				reply.NodeServices.Services = raw.([]*structs.NodeService)
+			}
+
+			return nil
+		})
+}
