@@ -10,11 +10,13 @@ import (
 	"time"
 
 	"github.com/hashicorp/consul/lib/semaphore"
+	"github.com/hashicorp/go-hclog"
 
 	"golang.org/x/time/rate"
 
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/connect"
+	"github.com/hashicorp/consul/agent/connect/ca"
 	"github.com/hashicorp/consul/agent/consul/state"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/go-memdb"
@@ -30,8 +32,10 @@ var (
 	// variable points to. Clients need to compare using `err.Error() ==
 	// consul.ErrRateLimited.Error()` which is very sad. Short of replacing our
 	// RPC mechanism it's hard to know how to make that much better though.
-	ErrConnectNotEnabled = errors.New("Connect must be enabled in order to use this endpoint")
-	ErrRateLimited       = errors.New("Rate limit reached, try again later")
+	ErrConnectNotEnabled    = errors.New("Connect must be enabled in order to use this endpoint")
+	ErrRateLimited          = errors.New("Rate limit reached, try again later")
+	ErrNotPrimaryDatacenter = errors.New("not the primary datacenter")
+	ErrStateReadOnly        = errors.New("CA Provider State is read-only")
 )
 
 const (
@@ -48,6 +52,8 @@ type ConnectCA struct {
 	// srv is a pointer back to the server.
 	srv *Server
 
+	logger hclog.Logger
+
 	// csrRateLimiter limits the rate of signing new certs if configured. Lazily
 	// initialized from current config to support dynamic changes.
 	// csrRateLimiterMu must be held while dereferencing the pointer or storing a
@@ -118,12 +124,12 @@ func (s *ConnectCA) ConfigurationGet(
 	if err != nil {
 		return err
 	}
-	if rule != nil && !rule.OperatorRead() {
+	if rule != nil && rule.OperatorRead(nil) != acl.Allow {
 		return acl.ErrPermissionDenied
 	}
 
 	state := s.srv.fsm.State()
-	_, config, err := state.CAConfig()
+	_, config, err := state.CAConfig(nil)
 	if err != nil {
 		return err
 	}
@@ -150,23 +156,40 @@ func (s *ConnectCA) ConfigurationSet(
 	if err != nil {
 		return err
 	}
-	if rule != nil && !rule.OperatorWrite() {
+	if rule != nil && rule.OperatorWrite(nil) != acl.Allow {
 		return acl.ErrPermissionDenied
 	}
 
 	// Exit early if it's a no-op change
 	state := s.srv.fsm.State()
-	confIdx, config, err := state.CAConfig()
+	confIdx, config, err := state.CAConfig(nil)
 	if err != nil {
 		return err
 	}
 
+	// Don't allow state changes. Either it needs to be empty or the same to allow
+	// read-modify-write loops that don't touch the State field.
+	if len(args.Config.State) > 0 &&
+		!reflect.DeepEqual(args.Config.State, config.State) {
+		return ErrStateReadOnly
+	}
+
 	// Don't allow users to change the ClusterID.
 	args.Config.ClusterID = config.ClusterID
 	if args.Config.Provider == config.Provider && reflect.DeepEqual(args.Config.Config, config.Config) {
 		return nil
 	}
 
+	// If the provider hasn't changed, we need to load the current Provider state
+	// so it can decide if it needs to change resources or not based on the config
+	// change.
+	if args.Config.Provider == config.Provider {
+		// Note this is a shallow copy since the State method doc requires the
+		// provider return a map that will not be further modified and should not
+		// modify the one we pass to Configure.
+		args.Config.State = config.State
+	}
+
 	// Create a new instance of the provider described by the config
 	// and get the current active root CA. This acts as a good validation
 	// of the config and makes sure the provider is functioning correctly
@@ -175,7 +198,15 @@ func (s *ConnectCA) ConfigurationSet(
 	if err != nil {
 		return fmt.Errorf("could not initialize provider: %v", err)
 	}
-	if err := newProvider.Configure(args.Config.ClusterID, true, args.Config.Config); err != nil {
+	pCfg := ca.ProviderConfig{
+		ClusterID:  args.Config.ClusterID,
+		Datacenter: s.srv.config.Datacenter,
+		// This endpoint can be called in a secondary DC too so set this correctly.
+		IsPrimary: s.srv.config.Datacenter == s.srv.config.PrimaryDatacenter,
+		RawConfig: args.Config.Config,
+		State:     args.Config.State,
+	}
+	if err := newProvider.Configure(pCfg); err != nil {
 		return fmt.Errorf("error configuring provider: %v", err)
 	}
 	if err := newProvider.GenerateRoot(); err != nil {
@@ -192,6 +223,13 @@ func (s *ConnectCA) ConfigurationSet(
 		return err
 	}
 
+	// See if the provider needs to persist any state along with the config
+	pState, err := newProvider.State()
+	if err != nil {
+		return fmt.Errorf("error getting provider state: %v", err)
+	}
+	args.Config.State = pState
+
 	// Compare the new provider's root CA ID to the current one. If they
 	// match, just update the existing provider with the new config.
 	// If they don't match, begin the root rotation process.
@@ -216,7 +254,7 @@ func (s *ConnectCA) ConfigurationSet(
 		// If the config has been committed, update the local provider instance
 		s.srv.setCAProvider(newProvider, newActiveRoot)
 
-		s.srv.logger.Printf("[INFO] connect: CA provider config updated")
+		s.logger.Info("CA provider config updated")
 
 		return nil
 	}
@@ -225,6 +263,26 @@ func (s *ConnectCA) ConfigurationSet(
 	// either by swapping the provider type or changing the provider's config
 	// to use a different root certificate.
 
+	// First up, sanity check that the current provider actually supports
+	// cross-signing.
+	oldProvider, _ := s.srv.getCAProvider()
+	if oldProvider == nil {
+		return fmt.Errorf("internal error: CA provider is nil")
+	}
+	canXSign, err := oldProvider.SupportsCrossSigning()
+	if err != nil {
+		return fmt.Errorf("CA provider error: %s", err)
+	}
+	if !canXSign && !args.Config.ForceWithoutCrossSigning {
+		return errors.New("The current CA Provider does not support cross-signing. " +
+			"You can try again with ForceWithoutCrossSigningSet but this may cause " +
+			"disruption - see documentation for more.")
+	}
+	if !canXSign && args.Config.ForceWithoutCrossSigning {
+		s.logger.Warn("current CA doesn't support cross signing but " +
+			"CA reconfiguration forced anyway with ForceWithoutCrossSigning")
+	}
+
 	// If it's a config change that would trigger a rotation (different provider/root):
 	// 1. Get the root from the new provider.
 	// 2. Call CrossSignCA on the old provider to sign the new root with the old one to
@@ -236,18 +294,18 @@ func (s *ConnectCA) ConfigurationSet(
 		return err
 	}
 
-	// Have the old provider cross-sign the new intermediate
-	oldProvider, _ := s.srv.getCAProvider()
-	if oldProvider == nil {
-		return fmt.Errorf("internal error: CA provider is nil")
-	}
-	xcCert, err := oldProvider.CrossSignCA(newRoot)
-	if err != nil {
-		return err
+	if canXSign {
+		// Have the old provider cross-sign the new root
+		xcCert, err := oldProvider.CrossSignCA(newRoot)
+		if err != nil {
+			return err
+		}
+
+		// Add the cross signed cert to the new CA's intermediates (to be attached
+		// to leaf certs).
+		newActiveRoot.IntermediateCerts = []string{xcCert}
 	}
 
-	// Add the cross signed cert to the new root's intermediates.
-	newActiveRoot.IntermediateCerts = []string{xcCert}
 	intermediate, err := newProvider.GenerateIntermediate()
 	if err != nil {
 		return err
@@ -293,10 +351,10 @@ func (s *ConnectCA) ConfigurationSet(
 	s.srv.setCAProvider(newProvider, newActiveRoot)
 
 	if err := oldProvider.Cleanup(); err != nil {
-		s.srv.logger.Printf("[WARN] connect: failed to clean up old provider %q", config.Provider)
+		s.logger.Warn("failed to clean up old provider", "provider", config.Provider)
 	}
 
-	s.srv.logger.Printf("[INFO] connect: CA rotated to new root under provider %q", args.Config.Provider)
+	s.logger.Info("CA rotated to new root under provider", "provider", args.Config.Provider)
 
 	return nil
 }
@@ -315,41 +373,25 @@ func (s *ConnectCA) Roots(
 		return ErrConnectNotEnabled
 	}
 
-	// Load the ClusterID to generate TrustDomain. We do this outside the loop
-	// since by definition this value should be immutable once set for lifetime of
-	// the cluster so we don't need to look it up more than once. We also don't
-	// have to worry about non-atomicity between the config fetch transaction and
-	// the CARoots transaction below since this field must remain immutable. Do
-	// not re-use this state/config for other logic that might care about changes
-	// of config during the blocking query below.
-	{
-		state := s.srv.fsm.State()
-		_, config, err := state.CAConfig()
-		if err != nil {
-			return err
-		}
-
-		// Check CA is actually bootstrapped...
-		if config != nil {
-			// Build TrustDomain based on the ClusterID stored.
-			signingID := connect.SpiffeIDSigningForCluster(config)
-			if signingID == nil {
-				// If CA is bootstrapped at all then this should never happen but be
-				// defensive.
-				return errors.New("no cluster trust domain setup")
-			}
-			reply.TrustDomain = signingID.Host()
-		}
-	}
-
 	return s.srv.blockingQuery(
 		&args.QueryOptions, &reply.QueryMeta,
 		func(ws memdb.WatchSet, state *state.Store) error {
-			index, roots, err := state.CARoots(ws)
+			index, roots, config, err := state.CARootsAndConfig(ws)
 			if err != nil {
 				return err
 			}
 
+			if config != nil {
+				// Build TrustDomain based on the ClusterID stored.
+				signingID := connect.SpiffeIDSigningForCluster(config)
+				if signingID == nil {
+					// If CA is bootstrapped at all then this should never happen but be
+					// defensive.
+					return errors.New("no cluster trust domain setup")
+				}
+				reply.TrustDomain = signingID.Host()
+			}
+
 			reply.Index, reply.Roots = index, roots
 			if reply.Roots == nil {
 				reply.Roots = make(structs.CARoots, 0)
@@ -374,6 +416,8 @@ func (s *ConnectCA) Roots(
 					IntermediateCerts:   r.IntermediateCerts,
 					RaftIndex:           r.RaftIndex,
 					Active:              r.Active,
+					PrivateKeyType:      r.PrivateKeyType,
+					PrivateKeyBits:      r.PrivateKeyBits,
 				}
 
 				if r.Active {
@@ -414,11 +458,13 @@ func (s *ConnectCA) Sign(
 	provider, caRoot := s.srv.getCAProvider()
 	if provider == nil {
 		return fmt.Errorf("internal error: CA provider is nil")
+	} else if caRoot == nil {
+		return fmt.Errorf("internal error: CA root is nil")
 	}
 
 	// Verify that the CSR entity is in the cluster's trust domain
 	state := s.srv.fsm.State()
-	_, config, err := state.CAConfig()
+	_, config, err := state.CAConfig(nil)
 	if err != nil {
 		return err
 	}
@@ -441,8 +487,12 @@ func (s *ConnectCA) Sign(
 	if err != nil {
 		return err
 	}
+	var authzContext acl.AuthorizerContext
+	var entMeta structs.EnterpriseMeta
 	if isService {
-		if rule != nil && !rule.ServiceWrite(serviceID.Service, nil) {
+		entMeta.Merge(serviceID.GetEnterpriseMeta())
+		entMeta.FillAuthzContext(&authzContext)
+		if rule != nil && rule.ServiceWrite(serviceID.Service, &authzContext) != acl.Allow {
 			return acl.ErrPermissionDenied
 		}
 
@@ -453,7 +503,8 @@ func (s *ConnectCA) Sign(
 				"we are %s", serviceID.Datacenter, s.srv.config.Datacenter)
 		}
 	} else if isAgent {
-		if rule != nil && !rule.NodeWrite(agentID.Agent, nil) {
+		structs.DefaultEnterpriseMeta().FillAuthzContext(&authzContext)
+		if rule != nil && rule.NodeWrite(agentID.Agent, &authzContext) != acl.Allow {
 			return acl.ErrPermissionDenied
 		}
 	}
@@ -482,6 +533,9 @@ func (s *ConnectCA) Sign(
 
 	// All seems to be in order, actually sign it.
 	pem, err := provider.Sign(csr)
+	if err == ca.ErrRateLimited {
+		return ErrRateLimited
+	}
 	if err != nil {
 		return err
 	}
@@ -537,10 +591,11 @@ func (s *ConnectCA) Sign(
 
 	// Set the response
 	*reply = structs.IssuedCert{
-		SerialNumber: connect.HexString(cert.SerialNumber.Bytes()),
-		CertPEM:      pem,
-		ValidAfter:   cert.NotBefore,
-		ValidBefore:  cert.NotAfter,
+		SerialNumber:   connect.EncodeSerialNumber(cert.SerialNumber),
+		CertPEM:        pem,
+		ValidAfter:     cert.NotBefore,
+		ValidBefore:    cert.NotAfter,
+		EnterpriseMeta: entMeta,
 		RaftIndex: structs.RaftIndex{
 			ModifyIndex: modIdx,
 			CreateIndex: modIdx,
@@ -556,3 +611,50 @@ func (s *ConnectCA) Sign(
 
 	return nil
 }
+
+// SignIntermediate signs an intermediate certificate for a remote datacenter.
+func (s *ConnectCA) SignIntermediate(
+	args *structs.CASignRequest,
+	reply *string) error {
+	// Exit early if Connect hasn't been enabled.
+	if !s.srv.config.ConnectEnabled {
+		return ErrConnectNotEnabled
+	}
+
+	if done, err := s.srv.forward("ConnectCA.SignIntermediate", args, args, reply); done {
+		return err
+	}
+
+	// Verify we are allowed to serve this request
+	if s.srv.config.PrimaryDatacenter != s.srv.config.Datacenter {
+		return ErrNotPrimaryDatacenter
+	}
+
+	// This action requires operator write access.
+	rule, err := s.srv.ResolveToken(args.Token)
+	if err != nil {
+		return err
+	}
+	if rule != nil && rule.OperatorWrite(nil) != acl.Allow {
+		return acl.ErrPermissionDenied
+	}
+
+	provider, _ := s.srv.getCAProvider()
+	if provider == nil {
+		return fmt.Errorf("internal error: CA provider is nil")
+	}
+
+	csr, err := connect.ParseCSR(args.CSR)
+	if err != nil {
+		return err
+	}
+
+	cert, err := provider.SignIntermediate(csr)
+	if err != nil {
+		return err
+	}
+
+	*reply = cert
+
+	return nil
+}
