@@ -8,7 +8,7 @@ import (
 	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/testrpc"
-	"github.com/hashicorp/net-rpc-msgpackrpc"
+	msgpackrpc "github.com/hashicorp/net-rpc-msgpackrpc"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -67,6 +67,7 @@ func TestIntentionApply_new(t *testing.T) {
 		actual.CreateIndex, actual.ModifyIndex = 0, 0
 		actual.CreatedAt = ixn.Intention.CreatedAt
 		actual.UpdatedAt = ixn.Intention.UpdatedAt
+		actual.Hash = ixn.Intention.Hash
 		ixn.Intention.UpdatePrecedence()
 		assert.Equal(ixn.Intention, actual)
 	}
@@ -222,6 +223,7 @@ func TestIntentionApply_updateGood(t *testing.T) {
 		actual.CreateIndex, actual.ModifyIndex = 0, 0
 		actual.CreatedAt = ixn.Intention.CreatedAt
 		actual.UpdatedAt = ixn.Intention.UpdatedAt
+		actual.Hash = ixn.Intention.Hash
 		ixn.Intention.UpdatePrecedence()
 		assert.Equal(ixn.Intention, actual)
 	}
@@ -381,11 +383,331 @@ service "foo" {
 		actual.CreateIndex, actual.ModifyIndex = 0, 0
 		actual.CreatedAt = ixn.Intention.CreatedAt
 		actual.UpdatedAt = ixn.Intention.UpdatedAt
+		actual.Hash = ixn.Intention.Hash
 		ixn.Intention.UpdatePrecedence()
 		assert.Equal(ixn.Intention, actual)
 	}
 }
 
+func TestIntention_WildcardACLEnforcement(t *testing.T) {
+	t.Parallel()
+
+	dir, srv := testACLServerWithConfig(t, nil, false)
+	defer os.RemoveAll(dir)
+	defer srv.Shutdown()
+	codec := rpcClient(t, srv)
+	defer codec.Close()
+
+	testrpc.WaitForLeader(t, srv.RPC, "dc1")
+
+	// create some test policies.
+
+	writeToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "" { policy = "deny" intentions = "write" }`)
+	require.NoError(t, err)
+	readToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "" { policy = "deny" intentions = "read" }`)
+	require.NoError(t, err)
+	exactToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service "*" { policy = "deny" intentions = "write" }`)
+	require.NoError(t, err)
+	wildcardPrefixToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "*" { policy = "deny" intentions = "write" }`)
+	require.NoError(t, err)
+	fooToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service "foo" { policy = "deny" intentions = "write" }`)
+	require.NoError(t, err)
+	denyToken, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service_prefix "" { policy = "deny" intentions = "deny" }`)
+	require.NoError(t, err)
+
+	doIntentionCreate := func(t *testing.T, token string, deny bool) string {
+		t.Helper()
+		ixn := structs.IntentionRequest{
+			Datacenter: "dc1",
+			Op:         structs.IntentionOpCreate,
+			Intention: &structs.Intention{
+				SourceNS:        "default",
+				SourceName:      "*",
+				DestinationNS:   "default",
+				DestinationName: "*",
+				Action:          structs.IntentionActionAllow,
+				SourceType:      structs.IntentionSourceConsul,
+			},
+			WriteRequest: structs.WriteRequest{Token: token},
+		}
+		var reply string
+		err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
+		if deny {
+			require.Error(t, err)
+			require.True(t, acl.IsErrPermissionDenied(err))
+			return ""
+		} else {
+			require.NoError(t, err)
+			require.NotEmpty(t, reply)
+			return reply
+		}
+	}
+
+	t.Run("deny-write-for-read-token", func(t *testing.T) {
+		// This tests ensures that tokens with only read access to all intentions
+		// cannot create a wildcard intention
+		doIntentionCreate(t, readToken.SecretID, true)
+	})
+
+	t.Run("deny-write-for-exact-wildcard-rule", func(t *testing.T) {
+		// This test ensures that having a rules like:
+		// service "*" {
+		//    intentions = "write"
+		// }
+		// will not actually allow creating an intention with a wildcard service name
+		doIntentionCreate(t, exactToken.SecretID, true)
+	})
+
+	t.Run("deny-write-for-prefix-wildcard-rule", func(t *testing.T) {
+		// This test ensures that having a rules like:
+		// service_prefix "*" {
+		//    intentions = "write"
+		// }
+		// will not actually allow creating an intention with a wildcard service name
+		doIntentionCreate(t, wildcardPrefixToken.SecretID, true)
+	})
+
+	var intentionID string
+	allowWriteOk := t.Run("allow-write", func(t *testing.T) {
+		// tests that a token with all the required privileges can create
+		// intentions with a wildcard destination
+		intentionID = doIntentionCreate(t, writeToken.SecretID, false)
+	})
+
+	requireAllowWrite := func(t *testing.T) {
+		t.Helper()
+		if !allowWriteOk {
+			t.Skip("Skipping because the allow-write subtest failed")
+		}
+	}
+
+	doIntentionRead := func(t *testing.T, token string, deny bool) {
+		t.Helper()
+		requireAllowWrite(t)
+		req := &structs.IntentionQueryRequest{
+			Datacenter:   "dc1",
+			IntentionID:  intentionID,
+			QueryOptions: structs.QueryOptions{Token: token},
+		}
+
+		var resp structs.IndexedIntentions
+		err := msgpackrpc.CallWithCodec(codec, "Intention.Get", req, &resp)
+		if deny {
+			require.Error(t, err)
+			require.True(t, acl.IsErrPermissionDenied(err))
+		} else {
+			require.NoError(t, err)
+			require.Len(t, resp.Intentions, 1)
+			require.Equal(t, "*", resp.Intentions[0].DestinationName)
+		}
+	}
+
+	t.Run("allow-read-for-write-token", func(t *testing.T) {
+		doIntentionRead(t, writeToken.SecretID, false)
+	})
+
+	t.Run("allow-read-for-read-token", func(t *testing.T) {
+		doIntentionRead(t, readToken.SecretID, false)
+	})
+
+	t.Run("allow-read-for-exact-wildcard-token", func(t *testing.T) {
+		// this is allowed because, the effect of the policy is to grant
+		// intention:write on the service named "*". When reading the
+		// intention we will validate that the token has read permissions
+		// for any intention that would match the wildcard.
+		doIntentionRead(t, exactToken.SecretID, false)
+	})
+
+	t.Run("allow-read-for-prefix-wildcard-token", func(t *testing.T) {
+		// this is allowed for the same reasons as for the
+		// exact-wildcard-token case
+		doIntentionRead(t, wildcardPrefixToken.SecretID, false)
+	})
+
+	t.Run("deny-read-for-deny-token", func(t *testing.T) {
+		doIntentionRead(t, denyToken.SecretID, true)
+	})
+
+	doIntentionList := func(t *testing.T, token string, deny bool) {
+		t.Helper()
+		requireAllowWrite(t)
+		req := &structs.DCSpecificRequest{
+			Datacenter:   "dc1",
+			QueryOptions: structs.QueryOptions{Token: token},
+		}
+
+		var resp structs.IndexedIntentions
+		err := msgpackrpc.CallWithCodec(codec, "Intention.List", req, &resp)
+		// even with permission denied this should return success but with an empty list
+		require.NoError(t, err)
+		if deny {
+			require.Empty(t, resp.Intentions)
+		} else {
+			require.Len(t, resp.Intentions, 1)
+			require.Equal(t, "*", resp.Intentions[0].DestinationName)
+		}
+	}
+
+	t.Run("allow-list-for-write-token", func(t *testing.T) {
+		doIntentionList(t, writeToken.SecretID, false)
+	})
+
+	t.Run("allow-list-for-read-token", func(t *testing.T) {
+		doIntentionList(t, readToken.SecretID, false)
+	})
+
+	t.Run("allow-list-for-exact-wildcard-token", func(t *testing.T) {
+		doIntentionList(t, exactToken.SecretID, false)
+	})
+
+	t.Run("allow-list-for-prefix-wildcard-token", func(t *testing.T) {
+		doIntentionList(t, wildcardPrefixToken.SecretID, false)
+	})
+
+	t.Run("deny-list-for-deny-token", func(t *testing.T) {
+		doIntentionList(t, denyToken.SecretID, true)
+	})
+
+	doIntentionMatch := func(t *testing.T, token string, deny bool) {
+		t.Helper()
+		requireAllowWrite(t)
+		req := &structs.IntentionQueryRequest{
+			Datacenter: "dc1",
+			Match: &structs.IntentionQueryMatch{
+				Type: structs.IntentionMatchDestination,
+				Entries: []structs.IntentionMatchEntry{
+					structs.IntentionMatchEntry{
+						Namespace: "default",
+						Name:      "*",
+					},
+				},
+			},
+			QueryOptions: structs.QueryOptions{Token: token},
+		}
+
+		var resp structs.IndexedIntentionMatches
+		err := msgpackrpc.CallWithCodec(codec, "Intention.Match", req, &resp)
+		if deny {
+			require.Error(t, err)
+			require.Empty(t, resp.Matches)
+		} else {
+			require.NoError(t, err)
+			require.Len(t, resp.Matches, 1)
+			require.Len(t, resp.Matches[0], 1)
+			require.Equal(t, "*", resp.Matches[0][0].DestinationName)
+		}
+	}
+
+	t.Run("allow-match-for-write-token", func(t *testing.T) {
+		doIntentionMatch(t, writeToken.SecretID, false)
+	})
+
+	t.Run("allow-match-for-read-token", func(t *testing.T) {
+		doIntentionMatch(t, readToken.SecretID, false)
+	})
+
+	t.Run("allow-match-for-exact-wildcard-token", func(t *testing.T) {
+		doIntentionMatch(t, exactToken.SecretID, false)
+	})
+
+	t.Run("allow-match-for-prefix-wildcard-token", func(t *testing.T) {
+		doIntentionMatch(t, wildcardPrefixToken.SecretID, false)
+	})
+
+	t.Run("deny-match-for-deny-token", func(t *testing.T) {
+		doIntentionMatch(t, denyToken.SecretID, true)
+	})
+
+	doIntentionUpdate := func(t *testing.T, token string, dest string, deny bool) {
+		t.Helper()
+		requireAllowWrite(t)
+		ixn := structs.IntentionRequest{
+			Datacenter: "dc1",
+			Op:         structs.IntentionOpUpdate,
+			Intention: &structs.Intention{
+				ID:              intentionID,
+				SourceNS:        "default",
+				SourceName:      "*",
+				DestinationNS:   "default",
+				DestinationName: dest,
+				Action:          structs.IntentionActionAllow,
+				SourceType:      structs.IntentionSourceConsul,
+			},
+			WriteRequest: structs.WriteRequest{Token: token},
+		}
+		var reply string
+		err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
+		if deny {
+			require.Error(t, err)
+			require.True(t, acl.IsErrPermissionDenied(err))
+		} else {
+			require.NoError(t, err)
+		}
+	}
+
+	t.Run("deny-update-for-foo-token", func(t *testing.T) {
+		doIntentionUpdate(t, fooToken.SecretID, "foo", true)
+	})
+
+	t.Run("allow-update-for-prefix-token", func(t *testing.T) {
+		// this tests that regardless of going from a wildcard intention
+		// to a non-wildcard or the opposite direction that the permissions
+		// are checked correctly. This also happens to leave the intention
+		// in a state ready for verifying similar things with deletion
+		doIntentionUpdate(t, writeToken.SecretID, "foo", false)
+		doIntentionUpdate(t, writeToken.SecretID, "*", false)
+	})
+
+	doIntentionDelete := func(t *testing.T, token string, deny bool) {
+		t.Helper()
+		requireAllowWrite(t)
+		ixn := structs.IntentionRequest{
+			Datacenter: "dc1",
+			Op:         structs.IntentionOpDelete,
+			Intention: &structs.Intention{
+				ID: intentionID,
+			},
+			WriteRequest: structs.WriteRequest{Token: token},
+		}
+		var reply string
+		err := msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply)
+		if deny {
+			require.Error(t, err)
+			require.True(t, acl.IsErrPermissionDenied(err))
+		} else {
+			require.NoError(t, err)
+		}
+	}
+
+	t.Run("deny-delete-for-read-token", func(t *testing.T) {
+		doIntentionDelete(t, readToken.SecretID, true)
+	})
+
+	t.Run("deny-delete-for-exact-wildcard-rule", func(t *testing.T) {
+		// This test ensures that having a rules like:
+		// service "*" {
+		//    intentions = "write"
+		// }
+		// will not actually allow deleting an intention with a wildcard service name
+		doIntentionDelete(t, exactToken.SecretID, true)
+	})
+
+	t.Run("deny-delete-for-prefix-wildcard-rule", func(t *testing.T) {
+		// This test ensures that having a rules like:
+		// service_prefix "*" {
+		//    intentions = "write"
+		// }
+		// will not actually allow creating an intention with a wildcard service name
+		doIntentionDelete(t, wildcardPrefixToken.SecretID, true)
+	})
+
+	t.Run("allow-delete", func(t *testing.T) {
+		// tests that a token with all the required privileges can delete
+		// intentions with a wildcard destination
+		doIntentionDelete(t, writeToken.SecretID, false)
+	})
+}
+
 // Test apply with delete and a default deny ACL
 func TestIntentionApply_aclDelete(t *testing.T) {
 	t.Parallel()
@@ -783,6 +1105,8 @@ service "foo" {
 			Op:         structs.IntentionOpCreate,
 			Intention:  structs.TestIntention(t),
 		}
+		ixn.Intention.SourceNS = "default"
+		ixn.Intention.DestinationNS = "default"
 		ixn.Intention.DestinationName = name
 		ixn.WriteRequest.Token = "root"
 
@@ -908,13 +1232,7 @@ func TestIntentionMatch_good(t *testing.T) {
 func TestIntentionMatch_acl(t *testing.T) {
 	t.Parallel()
 
-	assert := assert.New(t)
-	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
-		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
-	})
+	dir1, s1 := testACLServerWithConfig(t, nil, false)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
 	codec := rpcClient(t, s1)
@@ -922,37 +1240,15 @@ func TestIntentionMatch_acl(t *testing.T) {
 
 	testrpc.WaitForLeader(t, s1.RPC, "dc1")
 
-	// Create an ACL with service write permissions. This will grant
-	// intentions read.
-	var token string
-	{
-		var rules = `
-service "bar" {
-	policy = "write"
-}`
-
-		req := structs.ACLRequest{
-			Datacenter: "dc1",
-			Op:         structs.ACLSet,
-			ACL: structs.ACL{
-				Name:  "User token",
-				Type:  structs.ACLTokenTypeClient,
-				Rules: rules,
-			},
-			WriteRequest: structs.WriteRequest{Token: "root"},
-		}
-		assert.Nil(msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token))
-	}
+	token, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service "bar" { policy = "write" }`)
+	require.NoError(t, err)
 
 	// Create some records
 	{
-		insert := [][]string{
-			{"foo", "*"},
-			{"foo", "bar"},
-			{"foo", "baz"}, // shouldn't match
-			{"bar", "bar"}, // shouldn't match
-			{"bar", "*"},   // shouldn't match
-			{"*", "*"},
+		insert := []string{
+			"*",
+			"bar",
+			"baz",
 		}
 
 		for _, v := range insert {
@@ -961,13 +1257,12 @@ service "bar" {
 				Op:         structs.IntentionOpCreate,
 				Intention:  structs.TestIntention(t),
 			}
-			ixn.Intention.DestinationNS = v[0]
-			ixn.Intention.DestinationName = v[1]
-			ixn.WriteRequest.Token = "root"
+			ixn.Intention.DestinationName = v
+			ixn.WriteRequest.Token = TestDefaultMasterToken
 
 			// Create
 			var reply string
-			assert.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+			require.Nil(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 		}
 	}
 
@@ -979,7 +1274,7 @@ service "bar" {
 				Type: structs.IntentionMatchDestination,
 				Entries: []structs.IntentionMatchEntry{
 					{
-						Namespace: "foo",
+						Namespace: "default",
 						Name:      "bar",
 					},
 				},
@@ -987,8 +1282,8 @@ service "bar" {
 		}
 		var resp structs.IndexedIntentionMatches
 		err := msgpackrpc.CallWithCodec(codec, "Intention.Match", req, &resp)
-		assert.True(acl.IsErrPermissionDenied(err))
-		assert.Len(resp.Matches, 0)
+		require.True(t, acl.IsErrPermissionDenied(err))
+		require.Len(t, resp.Matches, 0)
 	}
 
 	// Test with proper token
@@ -999,24 +1294,24 @@ service "bar" {
 				Type: structs.IntentionMatchDestination,
 				Entries: []structs.IntentionMatchEntry{
 					{
-						Namespace: "foo",
+						Namespace: "default",
 						Name:      "bar",
 					},
 				},
 			},
-			QueryOptions: structs.QueryOptions{Token: token},
+			QueryOptions: structs.QueryOptions{Token: token.SecretID},
 		}
 		var resp structs.IndexedIntentionMatches
-		assert.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Match", req, &resp))
-		assert.Len(resp.Matches, 1)
+		require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Match", req, &resp))
+		require.Len(t, resp.Matches, 1)
 
-		expected := [][]string{{"foo", "bar"}, {"foo", "*"}, {"*", "*"}}
-		var actual [][]string
+		expected := []string{"bar", "*"}
+		var actual []string
 		for _, ixn := range resp.Matches[0] {
-			actual = append(actual, []string{ixn.DestinationNS, ixn.DestinationName})
+			actual = append(actual, ixn.DestinationName)
 		}
 
-		assert.Equal(expected, actual)
+		require.ElementsMatch(t, expected, actual)
 	}
 }
 
@@ -1179,13 +1474,7 @@ service "bar" {
 func TestIntentionCheck_match(t *testing.T) {
 	t.Parallel()
 
-	require := require.New(t)
-	dir1, s1 := testServerWithConfig(t, func(c *Config) {
-		c.ACLDatacenter = "dc1"
-		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
-	})
+	dir1, s1 := testACLServerWithConfig(t, nil, false)
 	defer os.RemoveAll(dir1)
 	defer s1.Shutdown()
 	codec := rpcClient(t, s1)
@@ -1193,33 +1482,15 @@ func TestIntentionCheck_match(t *testing.T) {
 
 	testrpc.WaitForLeader(t, s1.RPC, "dc1")
 
-	// Create an ACL with service read permissions. This will grant permission.
-	var token string
-	{
-		var rules = `
-service "bar" {
-	policy = "read"
-}`
-
-		req := structs.ACLRequest{
-			Datacenter: "dc1",
-			Op:         structs.ACLSet,
-			ACL: structs.ACL{
-				Name:  "User token",
-				Type:  structs.ACLTokenTypeClient,
-				Rules: rules,
-			},
-			WriteRequest: structs.WriteRequest{Token: "root"},
-		}
-		require.Nil(msgpackrpc.CallWithCodec(codec, "ACL.Apply", &req, &token))
-	}
+	token, err := upsertTestTokenWithPolicyRules(codec, TestDefaultMasterToken, "dc1", `service "api" { policy = "read" }`)
+	require.NoError(t, err)
 
 	// Create some intentions
 	{
 		insert := [][]string{
-			{"foo", "*", "foo", "*"},
-			{"foo", "*", "foo", "bar"},
-			{"bar", "*", "foo", "bar"}, // duplicate destination different source
+			{"web", "db"},
+			{"api", "db"},
+			{"web", "api"},
 		}
 
 		for _, v := range insert {
@@ -1227,18 +1498,17 @@ service "bar" {
 				Datacenter: "dc1",
 				Op:         structs.IntentionOpCreate,
 				Intention: &structs.Intention{
-					SourceNS:        v[0],
-					SourceName:      v[1],
-					DestinationNS:   v[2],
-					DestinationName: v[3],
+					SourceNS:        "default",
+					SourceName:      v[0],
+					DestinationNS:   "default",
+					DestinationName: v[1],
 					Action:          structs.IntentionActionAllow,
 				},
+				WriteRequest: structs.WriteRequest{Token: TestDefaultMasterToken},
 			}
-			ixn.WriteRequest.Token = "root"
-
 			// Create
 			var reply string
-			require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
+			require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Apply", &ixn, &reply))
 		}
 	}
 
@@ -1246,33 +1516,33 @@ service "bar" {
 	req := &structs.IntentionQueryRequest{
 		Datacenter: "dc1",
 		Check: &structs.IntentionQueryCheck{
-			SourceNS:        "foo",
-			SourceName:      "qux",
-			DestinationNS:   "foo",
-			DestinationName: "bar",
+			SourceNS:        "default",
+			SourceName:      "web",
+			DestinationNS:   "default",
+			DestinationName: "api",
 			SourceType:      structs.IntentionSourceConsul,
 		},
+		QueryOptions: structs.QueryOptions{Token: token.SecretID},
 	}
-	req.Token = token
 	var resp structs.IntentionQueryCheckResponse
-	require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Check", req, &resp))
-	require.True(resp.Allowed)
+	require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Check", req, &resp))
+	require.True(t, resp.Allowed)
 
 	// Test no match for sanity
 	{
 		req := &structs.IntentionQueryRequest{
 			Datacenter: "dc1",
 			Check: &structs.IntentionQueryCheck{
-				SourceNS:        "baz",
-				SourceName:      "qux",
-				DestinationNS:   "foo",
-				DestinationName: "bar",
+				SourceNS:        "default",
+				SourceName:      "db",
+				DestinationNS:   "default",
+				DestinationName: "api",
 				SourceType:      structs.IntentionSourceConsul,
 			},
+			QueryOptions: structs.QueryOptions{Token: token.SecretID},
 		}
-		req.Token = token
 		var resp structs.IntentionQueryCheckResponse
-		require.Nil(msgpackrpc.CallWithCodec(codec, "Intention.Check", req, &resp))
-		require.False(resp.Allowed)
+		require.NoError(t, msgpackrpc.CallWithCodec(codec, "Intention.Check", req, &resp))
+		require.False(t, resp.Allowed)
 	}
 }
