@@ -1213,3 +1213,25 @@ func TestLeader_ConfigEntryBootstrap_Fail(t *testing.T) {
 	result := <-ch
 	require.Empty(t, result)
 }
+
+func TestLeader_ACLLegacyReplication(t *testing.T) {
+	t.Parallel()
+
+	// This test relies on configuring a secondary DC with no route to the primary DC
+	// Having no route will cause the ACL mode checking of the primary to "fail". In this
+	// scenario legacy ACL replication should be enabled without also running new ACL
+	// replication routines.
+	cb := func(c *Config) {
+		c.Datacenter = "dc2"
+		c.ACLTokenReplication = true
+	}
+	dir, srv := testACLServerWithConfig(t, cb, true)
+	defer os.RemoveAll(dir)
+	defer srv.Shutdown()
+	waitForLeaderEstablishment(t, srv)
+
+	require.True(t, srv.leaderRoutineManager.IsRunning(legacyACLReplicationRoutineName))
+	require.False(t, srv.leaderRoutineManager.IsRunning(aclPolicyReplicationRoutineName))
+	require.False(t, srv.leaderRoutineManager.IsRunning(aclRoleReplicationRoutineName))
+	require.False(t, srv.leaderRoutineManager.IsRunning(aclTokenReplicationRoutineName))
+}
