@@ -63,6 +63,10 @@ func TestCoordinate_Datacenters(t *testing.T) {
 		t.Fatalf("err: %v", err)
 	}
 
+	if resp.Code != http.StatusOK {
+		t.Fatalf("bad: %v", resp.Code)
+	}
+
 	maps := obj.([]structs.DatacenterMap)
 	if len(maps) != 1 ||
 		maps[0].Datacenter != "dc1" ||
@@ -84,17 +88,21 @@ func TestCoordinate_Nodes(t *testing.T) {
 	retry.Run(t, func(r *retry.R) {
 		obj, err := a.srv.CoordinateNodes(resp, req)
 		if err != nil {
-			t.Fatalf("err: %v", err)
+			r.Fatalf("err: %v", err)
+		}
+
+		if resp.Code != http.StatusOK {
+			r.Fatalf("bad: %v", resp.Code)
 		}
 
 		// Check that coordinates are empty before registering a node
 		coordinates, ok := obj.(structs.Coordinates)
 		if !ok {
-			t.Fatalf("expected: structs.Coordinates, received: %+v", obj)
+			r.Fatalf("expected: structs.Coordinates, received: %+v", obj)
 		}
 
 		if len(coordinates) != 0 {
-			t.Fatalf("coordinates should be empty, received: %v", coordinates)
+			r.Fatalf("coordinates should be empty, received: %v", coordinates)
 		}
 	})
 
@@ -141,17 +149,21 @@ func TestCoordinate_Nodes(t *testing.T) {
 	retry.Run(t, func(r *retry.R) {
 		obj, err := a.srv.CoordinateNodes(resp, req)
 		if err != nil {
-			t.Fatalf("err: %v", err)
+			r.Fatalf("err: %v", err)
+		}
+
+		if resp.Code != http.StatusOK {
+			r.Fatalf("bad: %v", resp.Code)
 		}
 
 		coordinates, ok := obj.(structs.Coordinates)
 		if !ok {
-			t.Fatalf("expected: structs.Coordinates, received: %+v", obj)
+			r.Fatalf("expected: structs.Coordinates, received: %+v", obj)
 		}
 		if len(coordinates) != 2 ||
 			coordinates[0].Node != "bar" ||
 			coordinates[1].Node != "foo" {
-			t.Fatalf("expected: bar, foo recieved: %v", coordinates)
+			r.Fatalf("expected: bar, foo received: %v", coordinates)
 		}
 	})
 	// Filter on a nonexistent node segment
@@ -160,15 +172,19 @@ func TestCoordinate_Nodes(t *testing.T) {
 	retry.Run(t, func(r *retry.R) {
 		obj, err := a.srv.CoordinateNodes(resp, req)
 		if err != nil {
-			t.Fatalf("err: %v", err)
+			r.Fatalf("err: %v", err)
+		}
+
+		if resp.Code != http.StatusOK {
+			r.Fatalf("bad: %v", resp.Code)
 		}
 
 		coordinates, ok := obj.(structs.Coordinates)
 		if !ok {
-			t.Fatalf("expected: structs.Coordinates, received: %+v", obj)
+			r.Fatalf("expected: structs.Coordinates, received: %+v", obj)
 		}
 		if len(coordinates) != 0 {
-			t.Fatalf("coordinates should be empty, received: %v", coordinates)
+			r.Fatalf("coordinates should be empty, received: %v", coordinates)
 		}
 	})
 	// Filter on a real node segment
@@ -177,15 +193,19 @@ func TestCoordinate_Nodes(t *testing.T) {
 	retry.Run(t, func(r *retry.R) {
 		obj, err := a.srv.CoordinateNodes(resp, req)
 		if err != nil {
-			t.Fatalf("err: %v", err)
+			r.Fatalf("err: %v", err)
+		}
+
+		if resp.Code != http.StatusOK {
+			r.Fatalf("bad: %v", resp.Code)
 		}
 
 		coordinates, ok := obj.(structs.Coordinates)
 		if !ok {
-			t.Fatalf("expected: structs.Coordinates, received: %+v", obj)
+			r.Fatalf("expected: structs.Coordinates, received: %+v", obj)
 		}
 		if len(coordinates) != 1 || coordinates[0].Node != "foo" {
-			t.Fatalf("expected: foo received: %v", coordinates)
+			r.Fatalf("expected: foo received: %v", coordinates)
 		}
 	})
 	// Make sure the empty filter works
@@ -194,15 +214,15 @@ func TestCoordinate_Nodes(t *testing.T) {
 	retry.Run(t, func(r *retry.R) {
 		obj, err := a.srv.CoordinateNodes(resp, req)
 		if err != nil {
-			t.Fatalf("err: %v", err)
+			r.Fatalf("err: %v", err)
 		}
 
 		coordinates, ok := obj.(structs.Coordinates)
 		if !ok {
-			t.Fatalf("expected: structs.Coordinates, received: %+v", obj)
+			r.Fatalf("expected: structs.Coordinates, received: %+v", obj)
 		}
 		if len(coordinates) != 1 || coordinates[0].Node != "bar" {
-			t.Fatalf("expected: bar received: %v", coordinates)
+			r.Fatalf("expected: bar received: %v", coordinates)
 		}
 	})
 }
@@ -220,6 +240,7 @@ func TestCoordinate_Node(t *testing.T) {
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
+
 	if resp.Code != http.StatusNotFound {
 		t.Fatalf("bad: %v", resp.Code)
 	}
@@ -269,6 +290,10 @@ func TestCoordinate_Node(t *testing.T) {
 		t.Fatalf("err: %v", err)
 	}
 
+	if resp.Code != http.StatusOK {
+		t.Fatalf("bad: %v", resp.Code)
+	}
+
 	coordinates := obj.(structs.Coordinates)
 	if len(coordinates) != 1 ||
 		coordinates[0].Node != "foo" {
@@ -294,6 +319,10 @@ func TestCoordinate_Node(t *testing.T) {
 		t.Fatalf("err: %v", err)
 	}
 
+	if resp.Code != http.StatusOK {
+		t.Fatalf("bad: %v", resp.Code)
+	}
+
 	coordinates = obj.(structs.Coordinates)
 	if len(coordinates) != 1 || coordinates[0].Node != "foo" {
 		t.Fatalf("bad: %v", coordinates)
@@ -342,6 +371,11 @@ func TestCoordinate_Update(t *testing.T) {
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
+
+	if resp.Code != http.StatusOK {
+		t.Fatalf("bad: %v", resp.Code)
+	}
+
 	time.Sleep(300 * time.Millisecond)
 
 	// Query back and check the coordinates are present.
