@@ -41,6 +41,7 @@ type Server struct {
 	Status       serf.MemberStatus
 	NonVoter     bool
 	ACLs         structs.ACLMode
+	FeatureFlags map[string]int
 
 	// If true, use TLS when connecting to this server
 	UseTLS bool
@@ -103,6 +104,7 @@ func IsConsulServer(m serf.Member) (bool, *Server) {
 
 	segmentAddrs := make(map[string]string)
 	segmentPorts := make(map[string]int)
+	featureFlags := make(map[string]int)
 	for name, value := range m.Tags {
 		if strings.HasPrefix(name, "sl_") {
 			addr, port, err := net.SplitHostPort(value)
@@ -117,6 +119,13 @@ func IsConsulServer(m serf.Member) (bool, *Server) {
 			segmentName := strings.TrimPrefix(name, "sl_")
 			segmentAddrs[segmentName] = addr
 			segmentPorts[segmentName] = segmentPort
+		} else if strings.HasPrefix(name, "ft_") {
+			featureName := strings.TrimPrefix(name, "ft_")
+			featureState, err := strconv.Atoi(value)
+			if err != nil {
+				return false, nil
+			}
+			featureFlags[featureName] = featureState
 		}
 	}
 
@@ -173,6 +182,7 @@ func IsConsulServer(m serf.Member) (bool, *Server) {
 		UseTLS:       useTLS,
 		NonVoter:     nonVoter,
 		ACLs:         acls,
+		FeatureFlags: featureFlags,
 	}
 	return true, parts
 }
