@@ -235,6 +235,10 @@ func (m *Manager) FindServer() *metadata.Server {
 }
 
 func (m *Manager) checkServers(fn func(srv *metadata.Server) bool) bool {
+	if m == nil {
+		return true
+	}
+
 	for _, srv := range m.getServerList().servers {
 		if !fn(srv) {
 			return false
