@@ -1,7 +1,12 @@
 package structs
 
 import (
+	"flag"
 	"fmt"
+	"os"
+	"sort"
+	"strings"
+	"sync"
 	"testing"
 
 	"github.com/hashicorp/consul/api"
@@ -10,7 +15,7 @@ import (
 	"github.com/stretchr/testify/require"
 )
 
-const dumpFieldConfig bool = false
+var dumpFieldConfig = flag.Bool("dump-field-config", false, "generate field config dump file")
 
 ///////////////////////////////////////////////////////////////////////////////
 //
@@ -27,54 +32,127 @@ type fieldConfigTest struct {
 	expected bexpr.FieldConfigurations
 }
 
+// ----------------------------------------------------------------------------
+//
+// The following are not explicitly tested as they are supporting structures
+// nested within the other API responses
+//
+// ----------------------------------------------------------------------------
+
+var expectedFieldConfigServiceAddress bexpr.FieldConfigurations = bexpr.FieldConfigurations{
+	"Address": &bexpr.FieldConfiguration{
+		StructFieldName:     "Address",
+		CoerceFn:            bexpr.CoerceString,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
+	},
+	"Port": &bexpr.FieldConfiguration{
+		StructFieldName:     "Port",
+		CoerceFn:            bexpr.CoerceInt,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+	},
+}
+
+var expectedFieldConfigMeshGatewayConfig bexpr.FieldConfigurations = bexpr.FieldConfigurations{
+	"Mode": &bexpr.FieldConfiguration{
+		StructFieldName:     "Mode",
+		CoerceFn:            bexpr.CoerceString,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
+	},
+}
+
+var expectedFieldConfigExposeConfig bexpr.FieldConfigurations = bexpr.FieldConfigurations{
+	"Checks": &bexpr.FieldConfiguration{
+		StructFieldName:     "Checks",
+		CoerceFn:            bexpr.CoerceBool,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+	},
+	"Paths": &bexpr.FieldConfiguration{
+		StructFieldName:     "Paths",
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchIsEmpty, bexpr.MatchIsNotEmpty},
+		SubFields:           expectedFieldConfigPaths,
+	},
+}
+
+var expectedFieldConfigPaths bexpr.FieldConfigurations = bexpr.FieldConfigurations{
+	"ListenerPort": &bexpr.FieldConfiguration{
+		StructFieldName:     "ListenerPort",
+		CoerceFn:            bexpr.CoerceInt,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+	},
+	"Path": &bexpr.FieldConfiguration{
+		StructFieldName:     "Path",
+		CoerceFn:            bexpr.CoerceString,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
+	},
+	"LocalPathPort": &bexpr.FieldConfiguration{
+		StructFieldName:     "LocalPathPort",
+		CoerceFn:            bexpr.CoerceInt,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+	},
+	"Protocol": &bexpr.FieldConfiguration{
+		StructFieldName:     "Protocol",
+		CoerceFn:            bexpr.CoerceString,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
+	},
+	"ParsedFromCheck": &bexpr.FieldConfiguration{
+		StructFieldName:     "ParsedFromCheck",
+		CoerceFn:            bexpr.CoerceBool,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+	},
+}
+
 var expectedFieldConfigUpstreams bexpr.FieldConfigurations = bexpr.FieldConfigurations{
 	"DestinationType": &bexpr.FieldConfiguration{
 		StructFieldName:     "DestinationType",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"DestinationNamespace": &bexpr.FieldConfiguration{
 		StructFieldName:     "DestinationNamespace",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"DestinationName": &bexpr.FieldConfiguration{
 		StructFieldName:     "DestinationName",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Datacenter": &bexpr.FieldConfiguration{
 		StructFieldName:     "Datacenter",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"LocalBindAddress": &bexpr.FieldConfiguration{
 		StructFieldName:     "LocalBindAddress",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"LocalBindPort": &bexpr.FieldConfiguration{
 		StructFieldName:     "LocalBindPort",
 		CoerceFn:            bexpr.CoerceInt,
 		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
 	},
+	"MeshGateway": &bexpr.FieldConfiguration{
+		StructFieldName: "MeshGateway",
+		SubFields:       expectedFieldConfigMeshGatewayConfig,
+	},
 }
 
 var expectedFieldConfigConnectProxyConfig bexpr.FieldConfigurations = bexpr.FieldConfigurations{
 	"DestinationServiceName": &bexpr.FieldConfiguration{
 		StructFieldName:     "DestinationServiceName",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"DestinationServiceID": &bexpr.FieldConfiguration{
 		StructFieldName:     "DestinationServiceID",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"LocalServiceAddress": &bexpr.FieldConfiguration{
 		StructFieldName:     "LocalServiceAddress",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"LocalServicePort": &bexpr.FieldConfiguration{
 		StructFieldName:     "LocalServicePort",
@@ -86,6 +164,14 @@ var expectedFieldConfigConnectProxyConfig bexpr.FieldConfigurations = bexpr.Fiel
 		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchIsEmpty, bexpr.MatchIsNotEmpty},
 		SubFields:           expectedFieldConfigUpstreams,
 	},
+	"MeshGateway": &bexpr.FieldConfiguration{
+		StructFieldName: "MeshGateway",
+		SubFields:       expectedFieldConfigMeshGatewayConfig,
+	},
+	"Expose": &bexpr.FieldConfiguration{
+		StructFieldName: "Expose",
+		SubFields:       expectedFieldConfigExposeConfig,
+	},
 }
 
 var expectedFieldConfigServiceConnect bexpr.FieldConfigurations = bexpr.FieldConfigurations{
@@ -112,31 +198,43 @@ var expectedFieldConfigWeights bexpr.FieldConfigurations = bexpr.FieldConfigurat
 var expectedFieldConfigMapStringValue bexpr.FieldConfigurations = bexpr.FieldConfigurations{
 	bexpr.FieldNameAny: &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 }
 
-// these are not all in a table because some of them reference each other
+var expectedFieldConfigMapStringServiceAddress bexpr.FieldConfigurations = bexpr.FieldConfigurations{
+	bexpr.FieldNameAny: &bexpr.FieldConfiguration{
+		SubFields: expectedFieldConfigServiceAddress,
+	},
+}
+
+// ----------------------------------------------------------------------------
+//
+// The following structures are within the test table as they are structures
+// that will be sent back at the top level of API responses
+//
+// ----------------------------------------------------------------------------
+
 var expectedFieldConfigNode bexpr.FieldConfigurations = bexpr.FieldConfigurations{
 	"ID": &bexpr.FieldConfiguration{
 		StructFieldName:     "ID",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Node": &bexpr.FieldConfiguration{
 		StructFieldName:     "Node",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Address": &bexpr.FieldConfiguration{
 		StructFieldName:     "Address",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Datacenter": &bexpr.FieldConfiguration{
 		StructFieldName:     "Datacenter",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"TaggedAddresses": &bexpr.FieldConfiguration{
 		StructFieldName:     "TaggedAddresses",
@@ -145,7 +243,7 @@ var expectedFieldConfigNode bexpr.FieldConfigurations = bexpr.FieldConfiguration
 		SubFields: bexpr.FieldConfigurations{
 			bexpr.FieldNameAny: &bexpr.FieldConfiguration{
 				CoerceFn:            bexpr.CoerceString,
-				SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+				SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 			},
 		},
 	},
@@ -156,7 +254,7 @@ var expectedFieldConfigNode bexpr.FieldConfigurations = bexpr.FieldConfiguration
 		SubFields: bexpr.FieldConfigurations{
 			bexpr.FieldNameAny: &bexpr.FieldConfiguration{
 				CoerceFn:            bexpr.CoerceString,
-				SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+				SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 			},
 		},
 	},
@@ -166,17 +264,17 @@ var expectedFieldConfigNodeService bexpr.FieldConfigurations = bexpr.FieldConfig
 	"Kind": &bexpr.FieldConfiguration{
 		StructFieldName:     "Kind",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"ID": &bexpr.FieldConfiguration{
 		StructFieldName:     "ID",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Service": &bexpr.FieldConfiguration{
 		StructFieldName:     "Service",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Tags": &bexpr.FieldConfiguration{
 		StructFieldName:     "Tags",
@@ -186,7 +284,13 @@ var expectedFieldConfigNodeService bexpr.FieldConfigurations = bexpr.FieldConfig
 	"Address": &bexpr.FieldConfiguration{
 		StructFieldName:     "Address",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
+	},
+	"TaggedAddresses": &bexpr.FieldConfiguration{
+		StructFieldName:     "TaggedAddresses",
+		CoerceFn:            bexpr.CoerceString,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchIsEmpty, bexpr.MatchIsNotEmpty, bexpr.MatchIn, bexpr.MatchNotIn},
+		SubFields:           expectedFieldConfigMapStringServiceAddress,
 	},
 	"Meta": &bexpr.FieldConfiguration{
 		StructFieldName:     "Meta",
@@ -222,22 +326,22 @@ var expectedFieldConfigServiceNode bexpr.FieldConfigurations = bexpr.FieldConfig
 	"ID": &bexpr.FieldConfiguration{
 		StructFieldName:     "ID",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Node": &bexpr.FieldConfiguration{
 		StructFieldName:     "Node",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Address": &bexpr.FieldConfiguration{
 		StructFieldName:     "Address",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Datacenter": &bexpr.FieldConfiguration{
 		StructFieldName:     "Datacenter",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"TaggedAddresses": &bexpr.FieldConfiguration{
 		StructFieldName:     "TaggedAddresses",
@@ -254,17 +358,17 @@ var expectedFieldConfigServiceNode bexpr.FieldConfigurations = bexpr.FieldConfig
 	"ServiceKind": &bexpr.FieldConfiguration{
 		StructFieldName:     "ServiceKind",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"ServiceID": &bexpr.FieldConfiguration{
 		StructFieldName:     "ServiceID",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"ServiceName": &bexpr.FieldConfiguration{
 		StructFieldName:     "ServiceName",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"ServiceTags": &bexpr.FieldConfiguration{
 		StructFieldName:     "ServiceTags",
@@ -274,7 +378,13 @@ var expectedFieldConfigServiceNode bexpr.FieldConfigurations = bexpr.FieldConfig
 	"ServiceAddress": &bexpr.FieldConfiguration{
 		StructFieldName:     "ServiceAddress",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
+	},
+	"ServiceTaggedAddresses": &bexpr.FieldConfiguration{
+		StructFieldName:     "ServiceTaggedAddresses",
+		CoerceFn:            bexpr.CoerceString,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchIsEmpty, bexpr.MatchIsNotEmpty, bexpr.MatchIn, bexpr.MatchNotIn},
+		SubFields:           expectedFieldConfigMapStringServiceAddress,
 	},
 	"ServiceMeta": &bexpr.FieldConfiguration{
 		StructFieldName:     "ServiceMeta",
@@ -309,42 +419,42 @@ var expectedFieldConfigServiceNode bexpr.FieldConfigurations = bexpr.FieldConfig
 var expectedFieldConfigHealthCheck bexpr.FieldConfigurations = bexpr.FieldConfigurations{
 	"Node": &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 		StructFieldName:     "Node",
 	},
 	"CheckId": &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 		StructFieldName:     "CheckId",
 	},
 	"Name": &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 		StructFieldName:     "Name",
 	},
 	"Status": &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 		StructFieldName:     "Status",
 	},
 	"Notes": &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 		StructFieldName:     "Notes",
 	},
 	"Output": &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 		StructFieldName:     "Output",
 	},
 	"ServiceID": &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 		StructFieldName:     "ServiceID",
 	},
 	"ServiceName": &bexpr.FieldConfiguration{
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 		StructFieldName:     "ServiceName",
 	},
 	"ServiceTags": &bexpr.FieldConfiguration{
@@ -352,6 +462,11 @@ var expectedFieldConfigHealthCheck bexpr.FieldConfigurations = bexpr.FieldConfig
 		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchIsEmpty, bexpr.MatchIsNotEmpty, bexpr.MatchIn, bexpr.MatchNotIn},
 		StructFieldName:     "ServiceTags",
 	},
+	"Type": &bexpr.FieldConfiguration{
+		CoerceFn:            bexpr.CoerceString,
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
+		StructFieldName:     "Type",
+	},
 }
 
 var expectedFieldConfigCheckServiceNode bexpr.FieldConfigurations = bexpr.FieldConfigurations{
@@ -374,39 +489,29 @@ var expectedFieldConfigNodeInfo bexpr.FieldConfigurations = bexpr.FieldConfigura
 	"ID": &bexpr.FieldConfiguration{
 		StructFieldName:     "ID",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Node": &bexpr.FieldConfiguration{
 		StructFieldName:     "Node",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"Address": &bexpr.FieldConfiguration{
 		StructFieldName:     "Address",
 		CoerceFn:            bexpr.CoerceString,
-		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
+		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual, bexpr.MatchIn, bexpr.MatchNotIn, bexpr.MatchMatches, bexpr.MatchNotMatches},
 	},
 	"TaggedAddresses": &bexpr.FieldConfiguration{
 		StructFieldName:     "TaggedAddresses",
 		CoerceFn:            bexpr.CoerceString,
 		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchIsEmpty, bexpr.MatchIsNotEmpty, bexpr.MatchIn, bexpr.MatchNotIn},
-		SubFields: bexpr.FieldConfigurations{
-			bexpr.FieldNameAny: &bexpr.FieldConfiguration{
-				CoerceFn:            bexpr.CoerceString,
-				SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
-			},
-		},
+		SubFields:           expectedFieldConfigMapStringValue,
 	},
 	"Meta": &bexpr.FieldConfiguration{
 		StructFieldName:     "Meta",
 		CoerceFn:            bexpr.CoerceString,
 		SupportedOperations: []bexpr.MatchOperator{bexpr.MatchIsEmpty, bexpr.MatchIsNotEmpty, bexpr.MatchIn, bexpr.MatchNotIn},
-		SubFields: bexpr.FieldConfigurations{
-			bexpr.FieldNameAny: &bexpr.FieldConfiguration{
-				CoerceFn:            bexpr.CoerceString,
-				SupportedOperations: []bexpr.MatchOperator{bexpr.MatchEqual, bexpr.MatchNotEqual},
-			},
-		},
+		SubFields:           expectedFieldConfigMapStringValue,
 	},
 	"Services": &bexpr.FieldConfiguration{
 		StructFieldName:     "Services",
@@ -488,18 +593,93 @@ func validateFieldConfigurations(t *testing.T, expected, actual bexpr.FieldConfi
 	require.True(t, validateFieldConfigurationsRecurse(t, expected, actual, ""))
 }
 
+type fieldDumper struct {
+	fp   *os.File
+	lock sync.Mutex
+}
+
+func newFieldDumper(t *testing.T, path string) *fieldDumper {
+	fp, err := os.OpenFile(path, os.O_CREATE|os.O_WRONLY|os.O_TRUNC, 0660)
+	require.NoError(t, err)
+
+	return &fieldDumper{fp: fp}
+}
+
+func (d *fieldDumper) Close() {
+	d.fp.Close()
+}
+
+func (d *fieldDumper) DumpFields(name string, fields bexpr.FieldConfigurations) {
+	if d == nil {
+		return
+	}
+
+	selectorOps := make([][2]string, 0, 10)
+	// need at least 8 chars wide for "Selector"
+	maxSelectorLen := 8
+	// need at least 20 chars wid for "Supported Operaitons"
+	maxOpsLen := 20
+
+	fields.Walk(func(path bexpr.FieldPath, conf *bexpr.FieldConfiguration) bool {
+		if len(conf.SupportedOperations) < 1 {
+			return true
+		}
+
+		selector := path.String()
+		var ops []string
+		for _, op := range conf.SupportedOperations {
+			ops = append(ops, op.String())
+		}
+
+		opString := strings.Join(ops, ", ")
+		selLen := len(selector)
+		opsLen := len(opString)
+
+		if selLen > maxSelectorLen {
+			maxSelectorLen = selLen
+		}
+		if opsLen > maxOpsLen {
+			maxOpsLen = opsLen
+		}
+
+		selectorOps = append(selectorOps, [2]string{selector, opString})
+		return true
+	})
+
+	sort.Slice(selectorOps, func(i, j int) bool {
+		return selectorOps[i][0] < selectorOps[j][0]
+	})
+
+	d.lock.Lock()
+	defer d.lock.Unlock()
+
+	// this will print the header and the string form of the fields
+	fmt.Fprintf(d.fp, "===== %s =====\n%s\n\n", name, fields)
+
+	fmt.Fprintf(d.fp, "| %-[1]*[2]s | %-[3]*[4]s |\n", maxSelectorLen, "Selector", maxOpsLen, "Supported Operations")
+	fmt.Fprintf(d.fp, "| %s | %s |\n", strings.Repeat("-", maxSelectorLen), strings.Repeat("-", maxOpsLen))
+	for _, selOp := range selectorOps {
+		fmt.Fprintf(d.fp, "| %-[1]*[2]s | %-[3]*[4]s |\n", maxSelectorLen, selOp[0], maxOpsLen, selOp[1])
+	}
+	fmt.Fprintf(d.fp, "\n")
+}
+
 func TestStructs_FilterFieldConfigurations(t *testing.T) {
 	t.Parallel()
+
+	var d *fieldDumper
+	if *dumpFieldConfig {
+		d = newFieldDumper(t, "filter_fields.txt")
+		defer d.Close()
+	}
+
 	for name, tcase := range fieldConfigTests {
 		// capture these values in the closure
 		name := name
 		tcase := tcase
 		t.Run(name, func(t *testing.T) {
-			t.Parallel()
 			fields, err := bexpr.GenerateFieldConfigurations(tcase.dataType)
-			if dumpFieldConfig {
-				fmt.Printf("===== %s =====\n%s\n", name, fields)
-			}
+			d.DumpFields(name, fields)
 			require.NoError(t, err)
 			validateFieldConfigurations(t, tcase.expected, fields)
 		})
