@@ -3,9 +3,11 @@ package tokenupdate
 import (
 	"flag"
 	"fmt"
+	"strings"
 
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/command/acl"
+	"github.com/hashicorp/consul/command/acl/token"
 	"github.com/hashicorp/consul/command/flags"
 	"github.com/mitchellh/cli"
 )
@@ -34,6 +36,7 @@ type cmd struct {
 	mergeServiceIdents bool
 	showMeta           bool
 	upgradeLegacy      bool
+	format             string
 }
 
 func (c *cmd) init() {
@@ -66,6 +69,12 @@ func (c *cmd) init() {
 		"token to behave exactly like a new token but keep the same Secret.\n"+
 		"WARNING: you must ensure that the new policy or policies specified grant "+
 		"equivalent or appropriate access for the existing clients using this token.")
+	c.flags.StringVar(
+		&c.format,
+		"format",
+		token.PrettyFormat,
+		fmt.Sprintf("Output format {%s}", strings.Join(token.GetSupportedFormats(), "|")),
+	)
 
 	c.http = &flags.HTTPFlags{}
 	flags.Merge(c.flags, c.http.ClientFlags())
@@ -96,14 +105,14 @@ func (c *cmd) Run(args []string) int {
 		return 1
 	}
 
-	token, _, err := client.ACL().TokenRead(tokenID, nil)
+	t, _, err := client.ACL().TokenRead(tokenID, nil)
 	if err != nil {
 		c.UI.Error(fmt.Sprintf("Error when retrieving current token: %v", err))
 		return 1
 	}
 
 	if c.upgradeLegacy {
-		if token.Rules == "" {
+		if t.Rules == "" {
 			// This is just for convenience it should actually be harmless to allow it
 			// to go through anyway.
 			c.UI.Error(fmt.Sprintf("Can't use -upgrade-legacy on a non-legacy token"))
@@ -111,7 +120,7 @@ func (c *cmd) Run(args []string) int {
 		}
 		// Reset the rules to nothing forcing this to be updated as a non-legacy
 		// token but with same secret.
-		token.Rules = ""
+		t.Rules = ""
 	}
 
 	if c.description != "" {
@@ -121,7 +130,7 @@ func (c *cmd) Run(args []string) int {
 		// manually giving the new description. If it's a real issue we can always
 		// add another explicit `-remove-description` flag but it feels like an edge
 		// case that's not going to be critical to anyone.
-		token.Description = c.description
+		t.Description = c.description
 	}
 
 	parsedServiceIdents, err := acl.ExtractServiceIdentities(c.serviceIdents)
@@ -133,7 +142,7 @@ func (c *cmd) Run(args []string) int {
 	if c.mergePolicies {
 		for _, policyName := range c.policyNames {
 			found := false
-			for _, link := range token.Policies {
+			for _, link := range t.Policies {
 				if link.Name == policyName {
 					found = true
 					break
@@ -143,7 +152,7 @@ func (c *cmd) Run(args []string) int {
 			if !found {
 				// We could resolve names to IDs here but there isn't any reason why its would be better
 				// than allowing the agent to do it.
-				token.Policies = append(token.Policies, &api.ACLTokenPolicyLink{Name: policyName})
+				t.Policies = append(t.Policies, &api.ACLTokenPolicyLink{Name: policyName})
 			}
 		}
 
@@ -155,7 +164,7 @@ func (c *cmd) Run(args []string) int {
 			}
 			found := false
 
-			for _, link := range token.Policies {
+			for _, link := range t.Policies {
 				if link.ID == policyID {
 					found = true
 					break
@@ -163,16 +172,16 @@ func (c *cmd) Run(args []string) int {
 			}
 
 			if !found {
-				token.Policies = append(token.Policies, &api.ACLTokenPolicyLink{ID: policyID})
+				t.Policies = append(t.Policies, &api.ACLTokenPolicyLink{ID: policyID})
 			}
 		}
 	} else {
-		token.Policies = nil
+		t.Policies = nil
 
 		for _, policyName := range c.policyNames {
 			// We could resolve names to IDs here but there isn't any reason why its would be better
 			// than allowing the agent to do it.
-			token.Policies = append(token.Policies, &api.ACLTokenPolicyLink{Name: policyName})
+			t.Policies = append(t.Policies, &api.ACLTokenPolicyLink{Name: policyName})
 		}
 
 		for _, policyID := range c.policyIDs {
@@ -181,14 +190,14 @@ func (c *cmd) Run(args []string) int {
 				c.UI.Error(fmt.Sprintf("Error resolving policy ID %s: %v", policyID, err))
 				return 1
 			}
-			token.Policies = append(token.Policies, &api.ACLTokenPolicyLink{ID: policyID})
+			t.Policies = append(t.Policies, &api.ACLTokenPolicyLink{ID: policyID})
 		}
 	}
 
 	if c.mergeRoles {
 		for _, roleName := range c.roleNames {
 			found := false
-			for _, link := range token.Roles {
+			for _, link := range t.Roles {
 				if link.Name == roleName {
 					found = true
 					break
@@ -198,7 +207,7 @@ func (c *cmd) Run(args []string) int {
 			if !found {
 				// We could resolve names to IDs here but there isn't any reason why its would be better
 				// than allowing the agent to do it.
-				token.Roles = append(token.Roles, &api.ACLTokenRoleLink{Name: roleName})
+				t.Roles = append(t.Roles, &api.ACLTokenRoleLink{Name: roleName})
 			}
 		}
 
@@ -210,7 +219,7 @@ func (c *cmd) Run(args []string) int {
 			}
 			found := false
 
-			for _, link := range token.Roles {
+			for _, link := range t.Roles {
 				if link.ID == roleID {
 					found = true
 					break
@@ -218,16 +227,16 @@ func (c *cmd) Run(args []string) int {
 			}
 
 			if !found {
-				token.Roles = append(token.Roles, &api.ACLTokenRoleLink{Name: roleID})
+				t.Roles = append(t.Roles, &api.ACLTokenRoleLink{Name: roleID})
 			}
 		}
 	} else {
-		token.Roles = nil
+		t.Roles = nil
 
 		for _, roleName := range c.roleNames {
 			// We could resolve names to IDs here but there isn't any reason why its would be better
 			// than allowing the agent to do it.
-			token.Roles = append(token.Roles, &api.ACLTokenRoleLink{Name: roleName})
+			t.Roles = append(t.Roles, &api.ACLTokenRoleLink{Name: roleName})
 		}
 
 		for _, roleID := range c.roleIDs {
@@ -236,14 +245,14 @@ func (c *cmd) Run(args []string) int {
 				c.UI.Error(fmt.Sprintf("Error resolving role ID %s: %v", roleID, err))
 				return 1
 			}
-			token.Roles = append(token.Roles, &api.ACLTokenRoleLink{ID: roleID})
+			t.Roles = append(t.Roles, &api.ACLTokenRoleLink{ID: roleID})
 		}
 	}
 
 	if c.mergeServiceIdents {
 		for _, svcid := range parsedServiceIdents {
 			found := -1
-			for i, link := range token.ServiceIdentities {
+			for i, link := range t.ServiceIdentities {
 				if link.ServiceName == svcid.ServiceName {
 					found = i
 					break
@@ -251,23 +260,35 @@ func (c *cmd) Run(args []string) int {
 			}
 
 			if found != -1 {
-				token.ServiceIdentities[found] = svcid
+				t.ServiceIdentities[found] = svcid
 			} else {
-				token.ServiceIdentities = append(token.ServiceIdentities, svcid)
+				t.ServiceIdentities = append(t.ServiceIdentities, svcid)
 			}
 		}
 	} else {
-		token.ServiceIdentities = parsedServiceIdents
+		t.ServiceIdentities = parsedServiceIdents
 	}
 
-	token, _, err = client.ACL().TokenUpdate(token, nil)
+	t, _, err = client.ACL().TokenUpdate(t, nil)
 	if err != nil {
 		c.UI.Error(fmt.Sprintf("Failed to update token %s: %v", tokenID, err))
 		return 1
 	}
 
-	c.UI.Info("Token updated successfully.")
-	acl.PrintToken(token, c.UI, c.showMeta)
+	formatter, err := token.NewFormatter(c.format, c.showMeta)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+
+	out, err := formatter.FormatToken(t)
+	if err != nil {
+		c.UI.Error(err.Error())
+		return 1
+	}
+	if out != "" {
+		c.UI.Info(out)
+	}
 	return 0
 }
 
