@@ -1,6 +1,7 @@
 package tokenupdate
 
 import (
+	"encoding/json"
 	"os"
 	"strings"
 	"testing"
@@ -186,3 +187,63 @@ func TestTokenUpdateCommand(t *testing.T) {
 		assert.Equal(legacyToken.SecretID, gotToken.SecretID)
 	}
 }
+
+func TestTokenUpdateCommand_JSON(t *testing.T) {
+	t.Parallel()
+	assert := assert.New(t)
+	// Alias because we need to access require package in Retry below
+	req := require.New(t)
+
+	testDir := testutil.TempDir(t, "acl")
+	defer os.RemoveAll(testDir)
+
+	a := agent.NewTestAgent(t, t.Name(), `
+	primary_datacenter = "dc1"
+	acl {
+		enabled = true
+		tokens {
+			master = "root"
+		}
+	}`)
+
+	defer a.Shutdown()
+	testrpc.WaitForLeader(t, a.RPC, "dc1")
+
+	ui := cli.NewMockUi()
+
+	// Create a policy
+	client := a.Client()
+
+	policy, _, err := client.ACL().PolicyCreate(
+		&api.ACLPolicy{Name: "test-policy"},
+		&api.WriteOptions{Token: "root"},
+	)
+	req.NoError(err)
+
+	// create a token
+	token, _, err := client.ACL().TokenCreate(
+		&api.ACLToken{Description: "test"},
+		&api.WriteOptions{Token: "root"},
+	)
+	req.NoError(err)
+
+	t.Run("update with policy by name", func(t *testing.T) {
+		cmd := New(ui)
+		args := []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-id=" + token.AccessorID,
+			"-token=root",
+			"-policy-name=" + policy.Name,
+			"-description=test token",
+			"-format=json",
+		}
+
+		code := cmd.Run(args)
+		assert.Equal(code, 0)
+		assert.Empty(ui.ErrorWriter.String())
+
+		var jsonOutput json.RawMessage
+		err := json.Unmarshal([]byte(ui.OutputWriter.String()), &jsonOutput)
+		require.NoError(t, err, "token unmarshalling error")
+	})
+}
