@@ -22,6 +22,9 @@ type HTTPFlags struct {
 	// server flags
 	datacenter StringValue
 	stale      BoolValue
+
+	// namespace flags
+	namespace StringValue
 }
 
 func (f *HTTPFlags) ClientFlags() *flag.FlagSet {
@@ -55,7 +58,6 @@ func (f *HTTPFlags) ClientFlags() *flag.FlagSet {
 	fs.Var(&f.tlsServerName, "tls-server-name",
 		"The server name to use as the SNI host when connecting via TLS. This "+
 			"can also be specified via the CONSUL_TLS_SERVER_NAME environment variable.")
-
 	return fs
 }
 
@@ -72,6 +74,15 @@ func (f *HTTPFlags) ServerFlags() *flag.FlagSet {
 	return fs
 }
 
+func (f *HTTPFlags) NamespaceFlags() *flag.FlagSet {
+	fs := flag.NewFlagSet("", flag.ContinueOnError)
+	fs.Var(&f.namespace, "namespace",
+		"Specifies the namespace to query. If not provided, the namespace will be inferred "+
+			"from the request's ACL token, or will default to the `default` namespace. "+
+			"Namespaces is a Consul Enterprise feature.")
+	return fs
+}
+
 func (f *HTTPFlags) Addr() string {
 	return f.address.String()
 }
@@ -135,4 +146,5 @@ func (f *HTTPFlags) MergeOntoConfig(c *api.Config) {
 	f.keyFile.Merge(&c.TLSConfig.KeyFile)
 	f.tlsServerName.Merge(&c.TLSConfig.Address)
 	f.datacenter.Merge(&c.Datacenter)
+	f.namespace.Merge(&c.Namespace)
 }
