@@ -21,7 +21,7 @@ func TestKVExportCommand_noTabs(t *testing.T) {
 
 func TestKVExportCommand(t *testing.T) {
 	t.Parallel()
-	a := agent.NewTestAgent(t.Name(), ``)
+	a := agent.NewTestAgent(t, t.Name(), ``)
 	defer a.Shutdown()
 	client := a.Client()
 
