@@ -34,6 +34,7 @@ func TestOperatorAutopilotSetConfigCommand(t *testing.T) {
 		"-max-trailing-logs=99",
 		"-last-contact-threshold=123ms",
 		"-server-stabilization-time=123ms",
+		"-min-quorum=3",
 	}
 
 	code := c.Run(args)
@@ -65,4 +66,7 @@ func TestOperatorAutopilotSetConfigCommand(t *testing.T) {
 	if reply.ServerStabilizationTime != 123*time.Millisecond {
 		t.Fatalf("bad: %#v", reply)
 	}
+	if reply.MinQuorum != 3 {
+		t.Fatalf("bad: %#v", reply)
+	}
 }
