@@ -15,6 +15,7 @@ import (
 	"github.com/hashicorp/consul/api"
 	"github.com/hashicorp/consul/lib"
 	"github.com/hashicorp/consul/sdk/testutil"
+	"github.com/hashicorp/consul/sdk/testutil/retry"
 	"github.com/mitchellh/cli"
 	"github.com/stretchr/testify/require"
 )
@@ -152,6 +153,13 @@ func TestSnapshotSaveCommand_TruncatedStream(t *testing.T) {
 	}))
 	defer cleanup()
 
+	// Wait until the server is actually listening.
+	retry.Run(t, func(r *retry.R) {
+		resp, err := http.Get("http://" + fakeAddr + "/not-real")
+		require.NoError(r, err)
+		require.Equal(r, http.StatusNotFound, resp.StatusCode)
+	})
+
 	dir := testutil.TempDir(t, "snapshot")
 	defer os.RemoveAll(dir)
 
