@@ -86,10 +86,11 @@ func TestConfigurator_outgoingWrapper_OK(t *testing.T) {
 
 func TestConfigurator_outgoingWrapper_noverify_OK(t *testing.T) {
 	config := Config{
-		CAFile:   "../test/hostname/CertAuth.crt",
-		CertFile: "../test/hostname/Alice.crt",
-		KeyFile:  "../test/hostname/Alice.key",
-		Domain:   "consul",
+		VerifyOutgoing: true,
+		CAFile:         "../test/hostname/CertAuth.crt",
+		CertFile:       "../test/hostname/Alice.crt",
+		KeyFile:        "../test/hostname/Alice.key",
+		Domain:         "consul",
 	}
 
 	client, errc := startTLSServer(&config)
@@ -628,7 +629,7 @@ func TestConfigurator_OutgoingRPCTLSDisabled(t *testing.T) {
 		{false, true, nil, false},
 		{true, true, nil, false},
 
-		{false, false, &x509.CertPool{}, false},
+		// {false, false, &x509.CertPool{}, false},
 		{true, false, &x509.CertPool{}, false},
 		{false, true, &x509.CertPool{}, false},
 		{true, true, &x509.CertPool{}, false},
@@ -712,12 +713,23 @@ func TestConfigurator_OutgoingRPCConfig(t *testing.T) {
 }
 
 func TestConfigurator_OutgoingRPCWrapper(t *testing.T) {
-	c := Configurator{base: &Config{}, autoEncrypt: &autoEncrypt{}}
-	require.Nil(t, c.OutgoingRPCWrapper())
-	c.base.VerifyOutgoing = true
-	wrap := c.OutgoingRPCWrapper()
-	require.NotNil(t, wrap)
-	t.Log("TODO: actually call wrap here eventually")
+	c := &Configurator{base: &Config{}, autoEncrypt: &autoEncrypt{}}
+	wrapper := c.OutgoingRPCWrapper()
+	require.NotNil(t, wrapper)
+	conn := &net.TCPConn{}
+	cWrap, err := wrapper("", conn)
+	require.Equal(t, conn, cWrap)
+
+	c, err = NewConfigurator(Config{
+		VerifyOutgoing: true,
+		CAFile:         "../test/ca/root.cer",
+	}, nil)
+	require.NoError(t, err)
+
+	wrapper = c.OutgoingRPCWrapper()
+	require.NotNil(t, wrapper)
+	cWrap, err = wrapper("", conn)
+	require.NotEqual(t, conn, cWrap)
 }
 
 func TestConfigurator_UpdateChecks(t *testing.T) {
