@@ -1,5 +1,4 @@
 import { inject as service } from '@ember/service';
-import { get } from '@ember/object';
 import Component from '@ember/component';
 const append = function(content) {
   this.element.appendChild(content);
@@ -12,10 +11,10 @@ export default Component.extend({
   },
   didInsertElement: function() {
     this._super(...arguments);
-    get(this, 'buffer').on('add', this.append);
+    this.buffer.on('add', this.append);
   },
   didDestroyElement: function() {
     this._super(...arguments);
-    get(this, 'buffer').off('add', this.append);
+    this.buffer.off('add', this.append);
   },
 });
