@@ -1,5 +1,6 @@
 import Component from '@ember/component';
+import Slotted from 'block-slots';
 
-export default Component.extend({
+export default Component.extend(Slotted, {
   classNames: ['healthcheck-output'],
 });
