@@ -2,13 +2,12 @@ import Controller from '@ember/controller';
 import Component from '@ember/component';
 import Mixin from '@ember/object/mixin';
 import { inject as service } from '@ember/service';
-import { get } from '@ember/object';
 
 export default Mixin.create({
   dom: service('dom'),
   init: function() {
     this._super(...arguments);
-    this._listeners = get(this, 'dom').listeners();
+    this._listeners = this.dom.listeners();
     let teardown = ['willDestroy'];
     if (this instanceof Component) {
       teardown = ['willDestroyElement'];
