@@ -1,6 +1,5 @@
 import Mixin from '@ember/object/mixin';
 import { inject as service } from '@ember/service';
-import { get } from '@ember/object';
 import { assert } from '@ember/debug';
 export default Mixin.create({
   dom: service('dom'),
@@ -18,18 +17,14 @@ export default Mixin.create({
   },
   didInsertElement: function() {
     this._super(...arguments);
-    get(this, 'dom')
-      .viewport()
-      .addEventListener('resize', this.handler, false);
+    this.dom.viewport().addEventListener('resize', this.handler, false);
     this.didAppear();
   },
   didAppear: function() {
-    this.handler({ target: get(this, 'dom').viewport() });
+    this.handler({ target: this.dom.viewport() });
   },
   willDestroyElement: function() {
-    get(this, 'dom')
-      .viewport()
-      .removeEventListener('resize', this.handler, false);
+    this.dom.viewport().removeEventListener('resize', this.handler, false);
     this._super(...arguments);
   },
 });
