@@ -1,11 +1,10 @@
 import Model from 'ember-data/model';
 import attr from 'ember-data/attr';
-import writable from 'consul-ui/utils/model/writable';
 
 export const PRIMARY_KEY = 'uid';
 export const SLUG_KEY = 'ID';
 
-const model = Model.extend({
+export default Model.extend({
   [PRIMARY_KEY]: attr('string'),
   [SLUG_KEY]: attr('string'),
   Name: attr('string', {
@@ -21,6 +20,7 @@ const model = Model.extend({
   CreateTime: attr('date'),
   //
   Datacenter: attr('string'),
+  Namespace: attr('string'),
   Datacenters: attr(),
   CreateIndex: attr('number'),
   ModifyIndex: attr('number'),
@@ -29,5 +29,3 @@ const model = Model.extend({
     defaultValue: '',
   }),
 });
-export const ATTRS = writable(model, ['Name', 'Description', 'Rules', 'Datacenters']);
-export default model;
