@@ -1,11 +1,10 @@
 import Model from 'ember-data/model';
 import attr from 'ember-data/attr';
-import writable from 'consul-ui/utils/model/writable';
 
 export const PRIMARY_KEY = 'uid';
 export const SLUG_KEY = 'AccessorID';
 
-const model = Model.extend({
+export default Model.extend({
   [PRIMARY_KEY]: attr('string'),
   [SLUG_KEY]: attr('string'),
   IDPName: attr('string'),
@@ -22,6 +21,7 @@ const model = Model.extend({
     defaultValue: '',
   }),
   Datacenter: attr('string'),
+  Namespace: attr('string'),
   Local: attr('boolean'),
   Policies: attr({
     defaultValue: function() {
@@ -43,18 +43,3 @@ const model = Model.extend({
   CreateIndex: attr('number'),
   ModifyIndex: attr('number'),
 });
-// Name and Rules is only for legacy tokens
-export const ATTRS = writable(model, [
-  'Name',
-  'Rules',
-  'Type',
-  'Local',
-  'Description',
-  'Policies',
-  'Roles',
-  // SecretID isn't writable but we need it to identify an
-  // update via the old API, see TokenAdapter dataForRequest
-  'SecretID',
-  'AccessorID',
-]);
-export default model;
