@@ -1,21 +1,27 @@
 import Route from '@ember/routing/route';
 import { inject as service } from '@ember/service';
 import { hash } from 'rsvp';
-import { get } from '@ember/object';
+import { get, set } from '@ember/object';
 
 export default Route.extend({
   client: service('client/http'),
   repo: service('settings'),
   dcRepo: service('repository/dc'),
+  nspacesRepo: service('repository/nspace/disabled'),
   model: function(params) {
     return hash({
-      item: get(this, 'repo').findAll(),
-      dcs: get(this, 'dcRepo').findAll(),
+      item: this.repo.findAll(),
+      dcs: this.dcRepo.findAll(),
+      nspaces: this.nspacesRepo.findAll(),
+      nspace: this.nspacesRepo.getActive(),
     }).then(model => {
+      if (typeof get(model.item, 'client.blocking') === 'undefined') {
+        set(model, 'item.client', { blocking: true });
+      }
       return hash({
         ...model,
         ...{
-          dc: get(this, 'dcRepo').getActive(null, model.dcs),
+          dc: this.dcRepo.getActive(null, model.dcs),
         },
       });
     });
@@ -26,9 +32,9 @@ export default Route.extend({
   actions: {
     update: function(item) {
       if (!get(item, 'client.blocking')) {
-        get(this, 'client').abort();
+        this.client.abort();
       }
-      get(this, 'repo').persist(item);
+      this.repo.persist(item);
     },
   },
 });
