@@ -1,6 +1,5 @@
 import Service from '@ember/service';
 import { Promise } from 'rsvp';
-import { get } from '@ember/object';
 import getStorage from 'consul-ui/utils/storage/local-storage';
 const SCHEME = 'consul';
 const storage = getStorage(SCHEME);
@@ -10,7 +9,7 @@ export default Service.extend({
     // TODO: if possible this should be a promise
     // TODO: Actually this has nothing to do with settings it should be in the adapter,
     // which probably can't work with a promise based interface :(
-    const token = get(this, 'storage').getValue('token');
+    const token = this.storage.getValue('token');
     // TODO: The old UI always sent ?token=
     // replicate the old functionality here
     // but remove this to be cleaner if its not necessary
@@ -19,13 +18,13 @@ export default Service.extend({
     };
   },
   findAll: function(key) {
-    return Promise.resolve(get(this, 'storage').all());
+    return Promise.resolve(this.storage.all());
   },
   findBySlug: function(slug) {
-    return Promise.resolve(get(this, 'storage').getValue(slug));
+    return Promise.resolve(this.storage.getValue(slug));
   },
   persist: function(obj) {
-    const storage = get(this, 'storage');
+    const storage = this.storage;
     Object.keys(obj).forEach((item, i) => {
       storage.setValue(item, obj[item]);
     });
@@ -36,7 +35,7 @@ export default Service.extend({
     if (!Array.isArray(obj)) {
       obj = [obj];
     }
-    const storage = get(this, 'storage');
+    const storage = this.storage;
     const item = obj.reduce(function(prev, item, i, arr) {
       storage.removeValue(item);
       return prev;
