@@ -8,7 +8,7 @@ export default function(visitable, deletable, creatable, clickable, attribute, c
         acl: clickable('a'),
         actions: clickable('label'),
         use: clickable('[data-test-use]'),
-        confirmUse: clickable('button.type-delete'),
+        confirmUse: clickable('[data-test-confirm-use]'),
       })
     ),
     filter: filter,
